// -------------------------------------------------------------------------------- 
//!  @file  sfc.c                                                                   
//!  スーパーファミコンパッドソースファイル                                         
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include <stdio.h>
#include <string.h>
#include "pico/stdlib.h"
#include "hardware/spi.h"
#include "config.h"
#include "pad.h"
#include "sfc.h"


#ifdef  PAD_SFC_CFG_PIN_CLK

// -------------------------------------------------------------------------------- 
//!  1usec 待ち                                                                     
//!                                                                                 
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
static void RAMFUNC wait1us( void )
{
	volatile int	index ;


	for(index=0; index<11; index++) {
		asm volatile("nop") ;
	}
}


// -------------------------------------------------------------------------------- 
//!  ポート設定                                                                     
//!                                                                                 
//!  @param [in] gpio   GPIO 番号                                                   
//!  @param [in] value  値                                                          
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC setPort(uint gpio, bool value)
{
	gpio_put(gpio, value) ;

	wait1us() ;
}


// -------------------------------------------------------------------------------- 
//!  初期化                                                                         
//!                                                                                 
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
void pad_sfc_init( void )
{
	gpio_init( PAD_SFC_CFG_PIN_CLK ) ;
	gpio_init( PAD_SFC_CFG_PIN_PS ) ;
	gpio_init( PAD_SFC_CFG_PIN_DAT1 ) ;
	gpio_init( PAD_SFC_CFG_PIN_DAT2 ) ;
	gpio_init( PAD_SFC_CFG_PIN_SELECT ) ;

	gpio_pull_down( PAD_SFC_CFG_PIN_DAT1 ) ;
	gpio_pull_down( PAD_SFC_CFG_PIN_DAT2 ) ;

	gpio_put(PAD_SFC_CFG_PIN_CLK,        1) ;
	gpio_put(PAD_SFC_CFG_PIN_PS,         1) ;
	gpio_put(PAD_SFC_CFG_PIN_SELECT,     0) ;
	gpio_set_dir(PAD_SFC_CFG_PIN_CLK,    GPIO_OUT) ;
	gpio_set_dir(PAD_SFC_CFG_PIN_PS,     GPIO_OUT) ;
	gpio_set_dir(PAD_SFC_CFG_PIN_SELECT, GPIO_OUT) ;
	gpio_set_dir(PAD_SFC_CFG_PIN_DAT1,   GPIO_IN) ;
	gpio_set_dir(PAD_SFC_CFG_PIN_DAT2,   GPIO_IN) ;
}


// -------------------------------------------------------------------------------- 
//!  パッド状態を取得                                                               
//!                                                                                 
//!  @param [in,out] p_padInfo  パッド情報                                          
//!  @return                    なし                                                
// -------------------------------------------------------------------------------- 
void RAMFUNC pad_sfc_get( PADINFO *p_padInfo )
{
	uint16_t	status, readBit ;


	setPort(PAD_SFC_CFG_PIN_PS, 0) ;

	status  = 0 ;
	readBit = 1 ;

	do {
		if( !gpio_get( PAD_SFC_CFG_PIN_DAT1 ) ) {
			status |= readBit ;
		}

		setPort(PAD_SFC_CFG_PIN_CLK, 1) ;
		setPort(PAD_SFC_CFG_PIN_CLK, 0) ;

		readBit <<= 1 ;
	} while( readBit ) ;

	setPort(PAD_SFC_CFG_PIN_PS, 1) ;

	if( status != 0xffff ) {	// 接続

		p_padInfo->type   = PAD_ID_SFC ;
		p_padInfo->status = (((status >> 6) & 1) <<  PAD_LEFT_)   |
			                (((status >> 7) & 1) <<  PAD_RIGHT_)  |
			                (((status >> 4) & 1) <<  PAD_UP_)     |
			                (((status >> 5) & 1) <<  PAD_DOWN_)   |
			                (((status >> 2) & 1) <<  PAD_SELECT_) |
			                (((status >> 3) & 1) <<  PAD_START_)  |
			                (((status >> 0) & 1) << (PAD_BTN_+0)) |
			                (((status >> 8) & 1) << (PAD_BTN_+1)) |
			                (((status >> 1) & 1) << (PAD_BTN_+2)) |
			                (((status >> 9) & 1) << (PAD_BTN_+3)) |
			                (((status >>10) & 1) <<  PAD_LB_)     |
			                (((status >>11) & 1) <<  PAD_RB_) ;

	} else {					// 未接続

		p_padInfo->type   = PAD_TYPE_UNKNOWN ;
		p_padInfo->status = 0x0000 ;

	}
}

#endif//PAD_SFC_CFG_PIN_CLK
