// -------------------------------------------------------------------------------- 
//!  @file  vrc6.c                                                                  
//!  VRC6 音源 PCM データジェネレータソースファイル                                 
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include <stdio.h>
#include <string.h>
#include "pico/stdlib.h"
#include "config.h"
#include "vrc6.h"


#ifdef  PCMVRC6_CFG_BASECLK

	enum {
																	//!< クロック
		CLK							=	(PCMVRC6_CFG_BASECLK * PCMVRC6_CFG_TIME + (1000000 / 2)) / 1000000
	} ;


// -------------------------------------------------------------------------------- 
//!  更新                                                                           
//!                                                                                 
//!  @param [in,out] p  VRC6 情報                                                   
//!  @return            出力データ                                                  
// -------------------------------------------------------------------------------- 
int16_t RAMFUNC pcm_vrc6_update( PCMVRC6INFO *p )
{
	PCMVRC6CHANNELINFO	*p_channelInfo, *p_channelInfoEnd ;
	int32_t				output, freq, count ;
	uint32_t			update, reg, volume, duty, acc ;

	// チャンネル情報更新
	p_channelInfo    = &p->channelInfo[0] ;
	p_channelInfoEnd = &p->channelInfo[PCMVRC6_CHANNEL_COUNT] ;
	output           = 0 ;

	do {
		reg = p_channelInfo->reg[0] ;
		if( p_channelInfo->reg[1] != reg ) {
			update                = p_channelInfo->reg[1] ^ reg ;
			p_channelInfo->reg[1] = reg ;

			if( update & ((1<<31)|(1<<15)) ) {
				p_channelInfo->index = 0 ;
			}
		}

		freq  = (reg & 0xfff) + 1 ;
		count = p_channelInfo->count - CLK ;

		if( freq >= 8 ) {

			if( !(reg&(1<<31)) ) {		// パルス

				while( count <= 0 ) {
					count               += freq ;
					p_channelInfo->index = (p_channelInfo->index + 1) & 15 ;
				}

				volume = ((reg >> 16) & 15) * 16 ;
				duty   = (2 << ((reg >> 20) & 15)) - 1 ;

				if( !((duty>>p_channelInfo->index)&1) ) {
					volume = -volume ;
				}

			} else {					// ノコギリ

				acc = (reg >> 16) & 0x3f ;

				while( count <= 0 ) {
					count += freq ;

					if( ++p_channelInfo->index == 14 ) {
						p_channelInfo->index  = 0 ;
						p_channelInfo->volume = 0 ;
					} else
					if( !(p_channelInfo->index&1) && p_channelInfo->index ) {
						p_channelInfo->volume += acc ;
					}
				}

				volume = (p_channelInfo->volume - (acc * 7 / 2)) * 4 ;

			}

			p_channelInfo->count = (uint16_t )count ;

			output += volume ;
		}
	} while( ++p_channelInfo != p_channelInfoEnd ) ;

	return output ;
}

#endif//PCMVRC6_CFG_BASECLK
