// -------------------------------------------------------------------------------- 
//!  @file  vrc6.h                                                                  
//!  VRC6 音源 PCM データジェネレータヘッダファイル                                 
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#ifndef __PCM__VRC6_H__INCLUDED__
#define __PCM__VRC6_H__INCLUDED__


// レジスタ |0-------|MDDDVVVV|E---FFFF|FFFFFFFF|
//          |1-------|--AAAAAA|E---FFFF|FFFFFFFF|
//        S  0:パルス波, 1:鋸波
//        M  0:デューティー比 1:n, 1:デューティー比 1:1
//        D  デューティー比 n
//        V  音量
//        A  レート
//        E  0:チャンネル無効, 1:チャンネル有効


	enum {
		PCMVRC6_CHANNEL_COUNT		=	3,							//!< チャンネル数
	} ;

	//! チャンネル情報
	typedef struct {
		uint32_t					reg[2] ;						//!< レジスタ値
		uint8_t						index ;							//!< デューティー比・音量インデックス
		uint16_t					volume ;						//!< 音量
		uint16_t					count ;							//!< 周波数カウンタ
	} PCMVRC6CHANNELINFO ;

	//! VRC6 情報
	typedef struct {
		PCMVRC6CHANNELINFO			channelInfo[PCMVRC6_CHANNEL_COUNT] ;
																	//!< チャンネル情報
	} PCMVRC6INFO ;


extern int16_t RAMFUNC pcm_vrc6_update( PCMVRC6INFO *p ) ;


// -------------------------------------------------------------------------------- 
//!  リセット                                                                       
//!                                                                                 
//!  @param [out] p  VRC6 情報                                                      
//!  @return         なし                                                           
// -------------------------------------------------------------------------------- 
static inline void pcm_vrc6_reset( PCMVRC6INFO *p )
{
	memset(p, 0, sizeof( PCMVRC6INFO )) ;
}


// -------------------------------------------------------------------------------- 
//!  レジスタ設定                                                                   
//!                                                                                 
//!  @param [out] p        VRC6 情報                                                
//!  @param [in]  channel  チャンネル番号                                           
//!  @param [in]  value    レジスタ値                                               
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
static inline void pcm_vrc6_setReg(PCMVRC6INFO *p, uint8_t channel, uint32_t value)
{
	p->channelInfo[channel].reg[0] = value ;
}


#endif//__PCM__VRC6_H__INCLUDED__
