// -------------------------------------------------------------------------------- 
//!  @file  vga.h                                                                   
//!  VGA ヘッダファイル                                                             
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#ifndef __DRIVER__VGA_H__INCLUDED__
#define __DRIVER__VGA_H__INCLUDED__


#include "config.h"
#include "hardware/dma.h"

//! VGA 関数
#define VGAFUNC						__attribute__ ((section(".data.vga_funcs")))

// 背景レイヤー
#if     defined(VGA_CFG_BMP4) || defined(VGA_CFG_BMP8) || defined(VGA_CFG_BMP16) || defined(VGA_CFG_BGTILE)
#define VGABGLAYER
#endif//defined(VGA_CFG_BMP4) || defined(VGA_CFG_BMP8) || defined(VGA_CFG_BMP16) || defined(VGA_CFG_BGTILE)

// 前景レイヤー
#if     defined(VGA_CFG_FGTILE)
#define VGAFGLAYER
#endif//defined(VGA_CFG_FGTILE)


	enum {
		VGA_WIDTH					=	640,						//!< 水平解像度
		VGA_HEIGHT					=	480,						//!< 垂直解像度
		VGA_IMAGE_WIDTH				=	320,						//!< 画像幅
		VGA_IMAGE_HEIGHT			=	240							//!< 画像高さ
	} ;

	//! ビットマップ面
	enum {
		VGA_BMPH_WIDTH				=	320,						//!< ビットマップ面幅（高解像度）
		VGA_BMPL_WIDTH				=	256,						//!< ビットマップ面幅（低解像度）
		VGA_BMP_HEIGHT				=	240,						//!< ビットマップ面高さ
		VGA_BMP4_PALETTE_MAX		=	16,							//!< 4bit ビットマップ面パレット最大数
		VGA_BMP8_PALETTE_MAX		=	256							//!< 8bit ビットマップ面パレット最大数
	} ;

	//! 16bit ビットマップ面 VRAM
	enum {
		VGABMP16_OP_BIT				=	15,							//!< 透過ビット位置
		VGABMP16_OP_BITCOUNT		=	1,							//!< 透過ビット数
		VGABMP16_OP_BITMASK			=	(1 << VGABMP16_OP_BITCOUNT) - 1,
																	//!< 透過ビットマスク
		VGABMP16_OP					=	VGABMP16_OP_BITMASK << VGABMP16_OP_BIT
																	//!< 透過
	} ;

	//! タイル面
	enum {
		VGA_TILE_WIDTH				=	512,						//!< タイル面幅
		VGA_TILE_HEIGHT				=	512,						//!< タイル面高さ
		VGA_TILE_IMAGE_WIDTH		=	8,							//!< タイル面画像幅
		VGA_TILE_IMAGE_HEIGHT		=	8,							//!< タイル面画像高さ
		VGA_TILE_WIDTH_COUNT		=	VGA_TILE_WIDTH / VGA_TILE_IMAGE_WIDTH,
																	//!< タイル面１行あたりのキャラクタ数
		VGA_TILE_HEIGHT_COUNT		=	VGA_TILE_HEIGHT / VGA_TILE_IMAGE_HEIGHT,
																	//!< タイル面１列あたりのキャラクタ数
		VGA_TILE_PATTERN_MAX		=	1024,						//!< タイル面パターン定義最大数
		VGA_TILE_PALETTE_COUNT		=	16,							//!< タイル面１パレットあたりの色数
		VGA_TILE_PALETTE_MAX		=	16,							//!< タイル面パレット定義最大数
	} ;

	//! タイル面 VRAM
	enum {
		VGATILE_PC_BIT				=	12,							//!< パレットコードビット位置
		VGATILE_HF_BIT				=	11,							//!< 左右反転ビット位置
		VGATILE_VF_BIT				=	10,							//!< 上下反転ビット位置
		VGATILE_CC_BIT				=	0,							//!< キャラクターコードビット位置

		VGATILE_PC_BITCOUNT			=	4,							//!< パレットコードビット数
		VGATILE_HF_BITCOUNT			=	1,							//!< 左右反転ビット数
		VGATILE_VF_BITCOUNT			=	1,							//!< 上下反転ビット数
		VGATILE_CC_BITCOUNT			=	10,							//!< キャラクターコードビット数
																	//!  パレットコードマスク
		VGATILE_PC_BITMASK			=	(1 << VGATILE_PC_BITCOUNT) - 1,
		VGATILE_HF_BITMASK			=	(1 << VGATILE_HF_BITCOUNT) - 1,			//!< 左右反転マスク
		VGATILE_VF_BITMASK			=	(1 << VGATILE_VF_BITCOUNT) - 1,			//!< 上下反転マスク
		VGATILE_CC_BITMASK			=	(1 << VGATILE_CC_BITCOUNT) - 1,			//!< キャラクターコードマスク

		VGATILE_PC					=	VGATILE_PC_BITMASK << VGATILE_PC_BIT,	//!< パレットコード
		VGATILE_HF					=	VGATILE_HF_BITMASK << VGATILE_HF_BIT,	//!< 左右反転
		VGATILE_VF					=	VGATILE_VF_BITMASK << VGATILE_VF_BIT,	//!< 上下反転
		VGATILE_CC					=	VGATILE_CC_BITMASK << VGATILE_CC_BIT	//!< キャラクターコード
	} ;

#ifdef  VGA_CFG_SPR
	//! スプライト面
	enum {
		VGA_SPR_WIDTH				=	512,						//!< スプライト面幅
		VGA_SPR_HEIGHT				=	512,						//!< スプライト面高さ
		VGA_SPR_IMAGE_WIDTH			=	16,							//!< スプライト面画像幅
		VGA_SPR_IMAGE_HEIGHT		=	16,							//!< スプライト面画像高さ
		VGA_SPR_MAX					=	VGA_CFG_SPR_MAX,			//!< スプライト面最大数
		VGA_SPR_LINE_MAX			=	VGA_CFG_SPR_LINE_MAX,		//!< スプライト面１行あたりの表示最大数
		VGA_SPR_PATTERN_MAX			=	1024,						//!< スプライト面パターン定義最大数
		VGA_SPR_COLOR_COUNT			=	16,							//!< スプライト面１パレットあたりの色数
		VGA_SPR_PALETTE_MAX			=	16,							//!< スプライト面パレット定義最大数
	} ;

	//! スプライト面 VRAM
	enum {
		VGASPR_X_OFS_BIT			=	15,							//!< ｘ座標オフセットビット位置
		VGASPR_X_POS_BIT			=	0,							//!< ｘ座標ビット位置
		VGASPR_Y_OFS_BIT			=	15,							//!< ｙ座標オフセットビット位置
		VGASPR_Y_POS_BIT			=	0,							//!< ｙ座標ビット位置
		VGASPR_ATTR_PR_BIT			=	30,							//!< 優先順位ビット位置
		VGASPR_ATTR_PC_BIT			=	12,							//!< パレットコードビット位置
		VGASPR_ATTR_HF_BIT			=	11,							//!< 左右反転ビット位置
		VGASPR_ATTR_VF_BIT			=	10,							//!< 上下反転ビット位置
		VGASPR_ATTR_CC_BIT			=	0,							//!< キャラクターコードビット位置

		VGASPR_X_OFS_BITCOUNT		=	1,							//!< ｘ座標オフセットビット数
		VGASPR_X_POS_BITCOUNT		=	9,							//!< ｘ座標ビット数
		VGASPR_Y_OFS_BITCOUNT		=	1,							//!< ｙ座標オフセットビット数
		VGASPR_Y_POS_BITCOUNT		=	9,							//!< ｙ座標ビット数
		VGASPR_ATTR_PR_BITCOUNT		=	2,							//!< 優先順位ビット数
		VGASPR_ATTR_PC_BITCOUNT		=	4,							//!< パレットコードビット数
		VGASPR_ATTR_HF_BITCOUNT		=	1,							//!< 左右反転ビット数
		VGASPR_ATTR_VF_BITCOUNT		=	1,							//!< 上下反転ビット数
		VGASPR_ATTR_CC_BITCOUNT		=	10,							//!< キャラクターコードビット数

		VGASPR_X_OFS_BITMASK		=	(1 << VGASPR_X_OFS_BITCOUNT)   - 1,				//!< ｘ座標オフセットビットマスク
		VGASPR_X_POS_BITMASK		=	(1 << VGASPR_X_POS_BITCOUNT)   - 1,				//!< ｘ座標ビットマスク
		VGASPR_Y_OFS_BITMASK		=	(1 << VGASPR_Y_OFS_BITCOUNT)   - 1,				//!< ｙ座標オフセットビットマスク
		VGASPR_Y_POS_BITMASK		=	(1 << VGASPR_Y_POS_BITCOUNT)   - 1,				//!< ｙ座標ビットマスク
		VGASPR_ATTR_PR_BITMASK		=	(1 << VGASPR_ATTR_PR_BITCOUNT) - 1,				//!< 優先順位ビットマスク
		VGASPR_ATTR_PC_BITMASK		=	(1 << VGASPR_ATTR_PC_BITCOUNT) - 1,				//!< パレットコードビットマスク
		VGASPR_ATTR_HF_BITMASK		=	(1 << VGASPR_ATTR_HF_BITCOUNT) - 1,				//!< 左右反転ビットマスク
		VGASPR_ATTR_VF_BITMASK		=	(1 << VGASPR_ATTR_VF_BITCOUNT) - 1,				//!< 上下反転ビットマスク
		VGASPR_ATTR_CC_BITMASK		=	(1 << VGASPR_ATTR_CC_BITCOUNT) - 1,				//!< キャラクターコードビットマスク

		VGASPR_X_OFS				=	VGASPR_X_OFS_BITMASK   << VGASPR_X_OFS_BIT,		//!< ｘ座標オフセット
		VGASPR_X_POS				=	VGASPR_X_POS_BITMASK   << VGASPR_X_POS_BIT,		//!< ｘ座標
		VGASPR_Y_OFS				=	VGASPR_Y_OFS_BITMASK   << VGASPR_Y_OFS_BIT,		//!< ｙ座標オフセット
		VGASPR_Y_POS				=	VGASPR_Y_POS_BITMASK   << VGASPR_Y_POS_BIT,		//!< ｙ座標
		VGASPR_ATTR_PR				=	VGASPR_ATTR_PR_BITMASK << VGASPR_ATTR_PR_BIT,	//!< 優先順位
		VGASPR_ATTR_PC				=	VGASPR_ATTR_PC_BITMASK << VGASPR_ATTR_PC_BIT,	//!< パレットコード
		VGASPR_ATTR_HF				=	VGASPR_ATTR_HF_BITMASK << VGASPR_ATTR_HF_BIT,	//!< 左右反転
		VGASPR_ATTR_VF				=	VGASPR_ATTR_VF_BITMASK << VGASPR_ATTR_VF_BIT,	//!< 上下反転
		VGASPR_ATTR_CC				=	VGASPR_ATTR_CC_BITMASK << VGASPR_ATTR_CC_BIT	//!< キャラクターコード
	} ;

	//! スプライト面 VRAM データ
	typedef struct {
		uint16_t					x ;								//!< ｘ座標
		uint16_t					y ;								//!< ｙ座標
		uint32_t					attr ;							//!< 属性
	} VGASPR ;
#endif//VGA_CFG_SPR

	//! テキスト面
	enum {
		VGA_TXT_WIDTH				=	VGA_IMAGE_WIDTH,			//!< テキスト面幅
		VGA_TXT_HEIGHT				=	VGA_IMAGE_HEIGHT,			//!< テキスト面高さ
		VGA_TXT_IMAGE_WIDTH			=	8,							//!< テキスト面画像幅
		VGA_TXT_IMAGE_HEIGHT		=	8,							//!< テキスト面画像高さ
		VGA_TXT_WIDTH_COUNT			=	VGA_TXT_WIDTH  / VGA_TXT_IMAGE_WIDTH,
																	//!< テキスト面１行あたりのテキスト数
		VGA_TXT_HEIGHT_COUNT		=	VGA_TXT_HEIGHT / VGA_TXT_IMAGE_HEIGHT,
																	//!< テキスト面１列あたりのテキスト数
		VGA_TXT_PATTERN_MAX			=	1024,						//!< テキスト面パターン定義最大数
		VGA_TXT_PALETTE_COUNT		=	16,							//!< テキスト面パレットあたりの色数
		VGA_TXT_PALETTE_MAX			=	16							//!< テキスト面パレット定義最大数
	} ;

	//! テキスト VRAM
	enum {
		VGATXT_PC_BIT				=	12,							//!< パレットコードビット位置
		VGATXT_SH_BIT				=	11,							//!< シャドービット位置
		VGATXT_OP_BIT				=	10,							//!< 不透過ビット位置
		VGATXT_CC_BIT				=	0,							//!< キャラクターコードビット位置

		VGATXT_PC_BITCOUNT			=	4,							//!< パレットコードビット数
		VGATXT_SH_BITCOUNT			=	1,							//!< シャドービット数
		VGATXT_OP_BITCOUNT			=	1,							//!< 不透過ビット数
		VGATXT_CC_BITCOUNT			=	10,							//!< キャラクターコードビット数

		VGATXT_PC_BITMASK			=	(1 << VGATXT_PC_BITCOUNT) - 1,			//!< パレットコードビット数
		VGATXT_SH_BITMASK			=	(1 << VGATXT_SH_BITCOUNT) - 1,			//!< シャドービット数
		VGATXT_OP_BITMASK			=	(1 << VGATXT_OP_BITCOUNT) - 1,			//!< 不透過ビット数
		VGATXT_CC_BITMASK			=	(1 << VGATXT_CC_BITCOUNT) - 1,			//!< キャラクターコードビット数

		VGATXT_PC					=	VGATXT_PC_BITMASK << VGATXT_PC_BIT,		//!< パレットコードビット数
		VGATXT_SH					=	VGATXT_SH_BITMASK << VGATXT_SH_BIT,		//!< ブレンドビット数
		VGATXT_OP					=	VGATXT_OP_BITMASK << VGATXT_OP_BIT,		//!< 不透過ビット数
		VGATXT_CC					=	VGATXT_CC_BITMASK << VGATXT_CC_BIT		//!< キャラクターコードビット数
	} ;

	//! 背景モード
	enum {
		// 下位ビット
		VGA_BGMODE_320x240			=	0x00,						//!< 320x240
		VGA_BGMODE_256x240			=	0x01,						//!< 256x240
		VGA_BGMODE_STRETCH			=	0x02,						//!< ストレッチ

		// 上位ビット
		VGA_BGMODE_BLANK			=	0x00,						//!< 空白
		VGA_BGMODE_BMP4				=	0x10,						//!<  4bit ビットマップ面
		VGA_BGMODE_BMP8				=	0x20,						//!<  8bit ビットマップ面
		VGA_BGMODE_BMP16			=	0x30,						//!< 16bit ビットマップ面
		VGA_BGMODE_TILE				=	0x40						//!< タイル面
	} ;

//! HSync コールバック処理
typedef void (*VGAHSYNCPROC)( uint32_t line ) ;

#define VGALINEBUFF_MARGIN			16								//!< データ左右マージン

	//! ラインバッファ
	typedef struct {
		//! ページ
		struct {
			uint16_t				margin[VGALINEBUFF_MARGIN] ;	//!< 左右マージン
			uint16_t				data[VGA_IMAGE_WIDTH] ;			//!< データ
		} page[2] ;
		uint16_t					margin[VGALINEBUFF_MARGIN] ;	//!< 右マージン
	} VGALINEBUFF ;

	//! レイヤー
	typedef struct {
		uint16_t					x ; 							//!< ｘ座標オフセット
		uint16_t					y ;								//!< ｙ座標オフセット
		const uint8_t				*p_data ;						//!< データ
		const uint8_t				*p_palette ;					//!< パレットデータ
		const uint8_t				*p_pattern ;					//!< パターンデータ
	} VGALAYER ;

	//! VGA 情報
	typedef struct {
#ifdef  VGABGLAYER
		uint8_t						bgMode ;						//!< 背景モード
#endif//VGABGLAYER

		uint16_t					color ;							//!< 背景色
		uint16_t					dispLine ;						//!< 表示行
		volatile uint8_t			drawLine ;						//!< 描画行
		dma_channel_config			dmaConfigPixel ;				//!< ピクセル転送用 DMA チャンネルコンフィグ
		dma_channel_config			dmaConfigData ;					//!< データ転送用 DMA チャンネルコンフィグ
		VGAHSYNCPROC				hsyncProc ;						//!< 水平同期処理

#ifdef  VGABGLAYER
		VGALAYER					bgLayer ;						//!< 背景レイヤー
#endif//VGABGLAYER

#ifdef  VGAFGLAYER
		VGALAYER					fgLayer ;						//!< 前景レイヤー
#endif//VGAFGLAYER

#ifdef  VGA_CFG_SPR
		VGALAYER					sprLayer ;						//!< スプライトレイヤー
#endif//VGA_CFG_SPR

#ifdef  VGA_CFG_TXT
		VGALAYER					txtLayer ;						//!< テキストレイヤー
#endif//VGA_CFG_TXT

		VGALINEBUFF					lineBuff ;						//!< ラインバッファ
	} VGAINFO ;


extern VGAINFO						g_vgaInfo ;						//!< VGA 情報


extern void         vga_init( void ) ;
extern void         vga_start( void ) ;
extern void VGAFUNC vga_waitVSync( void ) ;


// -------------------------------------------------------------------------------- 
//!  HSync コールバックを設定                                                       
//!                                                                                 
//!  @param [in] hsyncProc  HSync コールバック処理                                  
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static inline void vga_setHSyncProc( VGAHSYNCPROC hsyncProc )
{
	g_vgaInfo.hsyncProc = hsyncProc ;
}


// -------------------------------------------------------------------------------- 
//!  背景色を設定                                                                   
//!                                                                                 
//!  @param [in] color  色                                                          
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static inline void vga_setColor( uint16_t color )
{
	g_vgaInfo.color = color ;
}


#ifdef  VGABGLAYER
// -------------------------------------------------------------------------------- 
//!  背景をブランクに設定                                                           
//!                                                                                 
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
static inline void vga_setBgBlank( void )
{
	g_vgaInfo.bgMode = VGA_BGMODE_BLANK ;
}
#endif//VGABGLAYER


#if     defined(VGA_CFG_BMP4) || defined(VGA_CFG_BMP8) || defined(VGA_CFG_BMP16)
// -------------------------------------------------------------------------------- 
//!  ビットマップ面の表示オフセットを設定                                           
//!                                                                                 
//!  @param [in] x  ｘ座標                                                          
//!  @param [in] y  ｙ座標                                                          
//!  @return        なし                                                            
// -------------------------------------------------------------------------------- 
static inline void vga_setBgBitmapOffset(int x, int y)
{
	if( y >= VGA_IMAGE_HEIGHT ) {
		y %= VGA_IMAGE_HEIGHT ;
	}

	g_vgaInfo.bgLayer.x = x ;
	g_vgaInfo.bgLayer.y = y ;
}
#endif//defined(VGA_CFG_BMP4) || defined(VGA_CFG_BMP8) || defined(VGA_CFG_BMP16)


#ifdef  VGA_CFG_BMP4
// -------------------------------------------------------------------------------- 
//!  背景に 4bit ビットマップ面を設定                                               
//!                                                                                 
//!  @param [in] p_data     4bit ビットマップデータアドレス                         
//!  @param [in] p_palette  パレットデータアドレス                                  
//!  @retval     true       正常終了                                                
//!  @retval     false      パラメータエラー                                        
// -------------------------------------------------------------------------------- 
static inline bool vga_setBgBitmap320x240x4(const uint32_t *p_data, const uint16_t *p_palette)
{
	if( (p_data==NULL) || (p_palette==NULL) ) {
		return false ;
	}

	g_vgaInfo.bgMode            = VGA_BGMODE_BMP4 ;
	g_vgaInfo.bgLayer.p_palette = (const uint8_t *)&p_palette[0] ;
	g_vgaInfo.bgLayer.p_data    = (const uint8_t *)&p_data[0] ;

	return true ;
}
#endif//VGA_CFG_BMP4


#ifdef  VGA_CFG_BMP8
// -------------------------------------------------------------------------------- 
//!  背景に 320x240 8bit ビットマップ面を設定                                       
//!                                                                                 
//!  @param [in] p_data     8bit ビットマップデータアドレス                         
//!  @param [in] p_palette  パレットデータアドレス                                  
//!  @retval     true       正常終了                                                
//!  @retval     false      パラメータエラー                                        
// -------------------------------------------------------------------------------- 
static inline bool vga_setBgBitmap320x240x8(const uint32_t *p_data, const uint16_t *p_palette)
{
	if( (p_data==NULL) || (p_palette==NULL) ) {
		return false ;
	}

	g_vgaInfo.bgMode            = VGA_BGMODE_BMP8 | VGA_BGMODE_320x240 ;
	g_vgaInfo.bgLayer.p_palette = (const uint8_t *)&p_palette[0] ;
	g_vgaInfo.bgLayer.p_data    = (const uint8_t *)&p_data[0] ;

	return true ;
}


// -------------------------------------------------------------------------------- 
//!  背景に 256x240 8bit ビットマップ面を設定                                       
//!                                                                                 
//!  @param [in] p_data     8bit ビットマップデータアドレス                         
//!  @param [in] p_palette  パレットデータアドレス                                  
//!  @retval     true       正常終了                                                
//!  @retval     false      パラメータエラー                                        
// -------------------------------------------------------------------------------- 
static inline bool vga_setBgBitmap256x240x8(const uint32_t *p_data, const uint16_t *p_palette, bool stretch)
{
	if( (p_data==NULL) || (p_palette==NULL) ) {
		return false ;
	}

	g_vgaInfo.bgMode = VGA_BGMODE_BMP8 | VGA_BGMODE_256x240 ;
	if( stretch != false ) {
		g_vgaInfo.bgMode = VGA_BGMODE_BMP8 | VGA_BGMODE_256x240 | VGA_BGMODE_STRETCH ;
	}

	g_vgaInfo.bgLayer.p_palette = (const uint8_t *)&p_palette[0] ;
	g_vgaInfo.bgLayer.p_data    = (const uint8_t *)&p_data[0] ;

	return true ;
}
#endif//VGA_CFG_BMP8


#ifdef  VGA_CFG_BMP16
// -------------------------------------------------------------------------------- 
//!  背景に 16bit ビットマップ面を設定                                              
//!                                                                                 
//!  @param [in] p_data  16bit ビットマップデータアドレス                           
//!  @retval     true    正常終了                                                   
//!  @retval     false   パラメータエラー                                           
// -------------------------------------------------------------------------------- 
static inline bool vga_setBgBitmap320x240x16( const uint32_t *p_data )
{
	if( p_data == NULL ) {
		return false ;
	}

	g_vgaInfo.bgMode         = VGA_BGMODE_BMP16 ;
	g_vgaInfo.bgLayer.p_data = (const uint8_t *)&p_data[0] ;

	return true ;
}
#endif//VGA_CFG_BMP16


#ifdef  VGA_CFG_BGTILE
// -------------------------------------------------------------------------------- 
//!  背景タイル面の表示オフセットを設定                                             
//!                                                                                 
//!  @param [in] x  ｘ座標                                                          
//!  @param [in] y  ｙ座標                                                          
//!  @return        なし                                                            
// -------------------------------------------------------------------------------- 
static inline void vga_setBgTileOffset(int x, int y)
{
	g_vgaInfo.bgLayer.x = x & (VGA_TILE_WIDTH  - 1) ;
	g_vgaInfo.bgLayer.y = y & (VGA_TILE_HEIGHT - 1) ;
}


// -------------------------------------------------------------------------------- 
//!  背景にタイル面を設定                                                           
//!                                                                                 
//!  @param [in] p_data     タイルデータアドレス                                    
//!  @param [in] p_pattern  パターンデータアドレス                                  
//!  @param [in] p_palette  パレットデータアドレス                                  
//!  @retval     true       正常終了                                                
//!  @retval     false      パラメータエラー                                        
// -------------------------------------------------------------------------------- 
static inline bool vga_setBgTile(const uint16_t *p_data, const uint32_t *p_pattern, const uint16_t *p_palette)
{
	if( (p_data==NULL) || (p_pattern==NULL) || (p_palette==NULL) ) {
		return false ;
	}

	g_vgaInfo.bgMode            = VGA_BGMODE_TILE ;
	g_vgaInfo.bgLayer.p_pattern = (const uint8_t *)&p_pattern[0] ;
	g_vgaInfo.bgLayer.p_palette = (const uint8_t *)&p_palette[0] ;
	g_vgaInfo.bgLayer.p_data    = (const uint8_t *)&p_data[0] ;

	return true ;
}
#endif//VGA_CFG_BGTILE


#ifdef  VGAFGLAYER
// -------------------------------------------------------------------------------- 
//!  前景にブランクを設定                                                           
//!                                                                                 
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
static inline void vga_setFgBlank( void )
{
	g_vgaInfo.fgLayer.p_data = NULL ;
}
#endif//VGAFGLAYER


#ifdef  VGA_CFG_FGTILE
// -------------------------------------------------------------------------------- 
//!  前景タイル面の表示オフセットを設定                                             
//!                                                                                 
//!  @param [in] x  ｘ座標                                                          
//!  @param [in] y  ｙ座標                                                          
//!  @return        なし                                                            
// -------------------------------------------------------------------------------- 
static inline void vga_setFgTileOffset(int x, int y)
{
	g_vgaInfo.fgLayer.x = x & (VGA_TILE_WIDTH  - 1) ;
	g_vgaInfo.fgLayer.y = y & (VGA_TILE_HEIGHT - 1) ;
}


// -------------------------------------------------------------------------------- 
//!  前景にタイル面を設定                                                           
//!                                                                                 
//!  @param [in] p_data     タイルデータアドレス                                    
//!  @param [in] p_pattern  パターンデータアドレス                                  
//!  @param [in] p_palette  パレットデータアドレス                                  
//!  @retval     true       正常終了                                                
//!  @retval     false      パラメータエラー                                        
// -------------------------------------------------------------------------------- 
static inline bool vga_setFgTile(const uint16_t *p_data, const uint32_t *p_pattern, const uint16_t *p_palette)
{
	if( (p_data==NULL) || (p_pattern==NULL) || (p_palette==NULL) ) {
		return false ;
	}

	g_vgaInfo.fgLayer.p_pattern = (const uint8_t *)&p_pattern[0] ;
	g_vgaInfo.fgLayer.p_palette = (const uint8_t *)&p_palette[0] ;
	g_vgaInfo.fgLayer.p_data    = (const uint8_t *)&p_data[0] ;

	return true ;
}
#endif//VGA_CFG_FGTILE


#ifdef  VGA_CFG_SPR
// -------------------------------------------------------------------------------- 
//!  スプライト面にブランクを設定                                                   
//!                                                                                 
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
static inline void vga_setSpriteBlank( void )
{
	g_vgaInfo.sprLayer.p_data = NULL ;
}


// -------------------------------------------------------------------------------- 
//!  スプライト面の表示オフセットを設定                                             
//!                                                                                 
//!  @param [in] x  ｘ座標                                                          
//!  @param [in] y  ｙ座標                                                          
//!  @return        なし                                                            
// -------------------------------------------------------------------------------- 
static inline void vga_setSpriteOffset(int x, int y)
{
	g_vgaInfo.sprLayer.x = x & (VGA_SPR_WIDTH  - 1) ;
	g_vgaInfo.sprLayer.y = y & (VGA_SPR_HEIGHT - 1) ;
}


// -------------------------------------------------------------------------------- 
//!  スプライト面に設定                                                             
//!                                                                                 
//!  @param [in] p_spr      スプライトデータアドレス                                
//!  @param [in] p_pattern  パターンデータアドレス                                  
//!  @param [in] p_palette  パレットデータアドレス                                  
//!  @retval     true       正常終了                                                
//!  @retval     false      パラメータエラー                                        
// -------------------------------------------------------------------------------- 
static inline bool vga_setSprite(const VGASPR *p_spr, const uint32_t *p_pattern, const uint16_t *p_palette)
{
	if( (p_spr==NULL) || (p_pattern==NULL) || (p_palette==NULL) ) {
		return false ;
	}

	g_vgaInfo.sprLayer.p_pattern  = (const uint8_t *)&p_pattern[0] ;
	g_vgaInfo.sprLayer.p_palette  = (const uint8_t *)&p_palette[0] ;
	g_vgaInfo.sprLayer.p_data     = (const uint8_t *)&p_spr[0] ;

	return true ;
}
#endif//VGA_CFG_SPR


#ifdef  VGA_CFG_TXT
// -------------------------------------------------------------------------------- 
//!  テキスト面にブランクを設定                                                     
//!                                                                                 
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
static inline void vga_setTextBlank( void )
{
	g_vgaInfo.txtLayer.p_data = NULL ;
}


// -------------------------------------------------------------------------------- 
//!  テキスト面の表示オフセットを設定                                               
//!                                                                                 
//!  @param [in] x  ｘ座標                                                          
//!  @param [in] y  ｙ座標                                                          
//!  @return        なし                                                            
// -------------------------------------------------------------------------------- 
static inline void vga_setTextOffset(int x, int y)
{
	if( (uint )y >= VGA_IMAGE_HEIGHT ) {
		y = (uint )y % VGA_IMAGE_HEIGHT ;
	}

	g_vgaInfo.txtLayer.x = x ;
	g_vgaInfo.txtLayer.y = y ;
}


// -------------------------------------------------------------------------------- 
//!  テキスト面を前景に設定                                                         
//!                                                                                 
//!  @param [in] p_txt      テキストデータアドレス                                  
//!  @param [in] p_pattern  パターンデータアドレス                                  
//!  @param [in] p_palette  パレットデータアドレス                                  
//!  @retval     true       正常終了                                                
//!  @retval     false      パラメータエラー                                        
// -------------------------------------------------------------------------------- 
static inline bool vga_setText(const uint16_t *p_txt, const uint32_t *p_pattern, const uint16_t *p_palette)
{
	if( (p_txt==NULL) || (p_pattern==NULL) || (p_palette==NULL) ) {
		return false ;
	}

	g_vgaInfo.txtLayer.p_pattern  = (const uint8_t *)&p_pattern[0] ;
	g_vgaInfo.txtLayer.p_palette  = (const uint8_t *)&p_palette[0] ;
	g_vgaInfo.txtLayer.p_data     = (const uint8_t *)&p_txt[0] ;

	return true ;
}
#endif//VGA_CFG_TXT


#endif//__DRIVER__VGA_H__INCLUDED__
