// -------------------------------------------------------------------------------- 
//!  @file  z80.h                                                                   
//!  Z80 のヘッダーファイル                                                         
//                                                                                  
//   Copyright (C) 2022 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#ifndef __Z80_H__INCLUDED__
#define __Z80_H__INCLUDED__

	//! Z80
	typedef struct {
		uint16_t					pc ;							//!< プログラムカウンタ
		uint8_t						status ;						//!< 状態
		uint8_t						r ;								//!< R レジスタ下位 7bit

		union {														//!< レジスタ
			struct {
				uint8_t				f ;								//!< F
				uint8_t				a ;								//!< A
				uint8_t				c ;								//!< C
				uint8_t				b ;								//!< B
				uint8_t				e ;								//!< E
				uint8_t				d ;								//!< D
				uint8_t				l ;								//!< L
				uint8_t				h ;								//!< H
				uint8_t				f_ ;							//!< F'
				uint8_t				a_ ;							//!< A'
				uint8_t				c_ ;							//!< C'
				uint8_t				b_ ;							//!< B'
				uint8_t				e_ ;							//!< E'
				uint8_t				d_ ;							//!< D'
				uint8_t				l_ ;							//!< L'
				uint8_t				h_ ;							//!< H'
				uint8_t				ixl ;							//!< IXL
				uint8_t				ixh ;							//!< IXH
				uint8_t				iyl ;							//!< IYL
				uint8_t				iyh ;							//!< IYH
				uint8_t				spl ;							//!< SPL
				uint8_t				sph ;							//!< SPH
			} b ;
			struct {
				uint16_t			af ;							//!< AF
				uint16_t			bc ;							//!< BC
				uint16_t			de ;							//!< DE
				uint16_t			hl ;							//!< HL
				uint16_t			af_ ;							//!< AF'
				uint16_t			bc_ ;							//!< BC'
				uint16_t			de_ ;							//!< DE'
				uint16_t			hl_ ;							//!< HL'
				uint16_t			ix ;							//!< IX
				uint16_t			iy ;							//!< IY
				uint16_t			sp ;							//!< SP
			} w ;
		} regs ;

		uint8_t						rt ;							//!< R レジスタ最上位ビット
		uint8_t						i ;								//!< I レジスタ
		uint8_t						im ;							//!< マスカブル割り込みモード
		uint8_t						iff ;							//!< 割り込みイネーブルフリップフロップ
		uint16_t					calc ;							//!< 内部計算用
		PIN							pinNmi ;						//!< NMI ピン
		PIN							pinInt ;						//!< INT ピン
		ADDRESSBUS					*p_io;							//!< I/O アドレスバス
		ADDRESSBUS					*p_mem ;						//!< メモリーアドレスバス
		CLOCKEVENT					clockEvent ;					//!< クロックイベント
		int16_t						pinNmiValue ;					//!< NMI 値
		int16_t						pinIntValue ;					//!< INT 値
		uint8_t						flags[256] ;					//!< フラグ
	} Z80 ;


extern void z80_init(Z80*p, ADDRESSBUS *p_io, ADDRESSBUS *p_mem) ;
extern void z80_reset( Z80 *p ) ;


#endif//__Z80_H__INCLUDED__
