// -------------------------------------------------------------------------------- 
//!  @file  bullet00.c                                                              
//!  弾 00 タスク処理のソースファイル                                               
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "enemy.h"
#include "bullet00.h"


// -------------------------------------------------------------------------------- 
//!  弾 00 当たり処理                                                               
//!                                                                                 
//!  @param [in,out] p_self   自分                                                  
//!  @param [in,out] p_rival  相手                                                  
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static void RAMFUNC bullet00_hit(HIT *p_self, HIT *p_rival)
{
	ENEMYTCB	*p_tcb=MEMBERSTRUCT(ENEMYTCB, obj.hit, p_self) ;


	p_self->flag = 0 ;

	p_rival->proc(p_rival, p_self) ;

	p_tcb->taskProc = enemy_destroy ;
}


// -------------------------------------------------------------------------------- 
//!  弾 00 タスク                                                                   
//!                                                                                 
//!  @param [in] p_tcb  敵タスク TCB                                                
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC bullet01( ENEMYTCB *p_tcb )
{
	// ｘ方向移動
	p_tcb->obj.x += getInt( getCos( p_tcb->angle ) * p_tcb->speed ) - toFp( g.bg.xofs ) ;
	if( (getInt( p_tcb->obj.x )<-8) || (getInt( p_tcb->obj.x )>=(SCREEN_WIDTH+8)) ) {
		enemy_destroy( p_tcb ) ;
		return ;
	}

	// ｙ方向移動
	p_tcb->obj.y += getInt( getSin( p_tcb->angle ) * p_tcb->speed ) - toFp( g.bg.yofs ) ;
	if( (getInt( p_tcb->obj.y )<-8) || (getInt( p_tcb->obj.y )>=(SCREEN_HEIGHT+8)) ) {
		enemy_destroy( p_tcb ) ;
		return ;
	}

	// 壁
	if( bg_checkHit(p_tcb->obj.x, p_tcb->obj.y) & BG_ATTR_WALL ) {
		enemy_destroy( p_tcb ) ;
		return ;
	}

	obj_updateAction( &p_tcb->obj ) ;
}


// -------------------------------------------------------------------------------- 
//!  弾 00 タスクを生成                                                             
//!                                                                                 
//!  @param [in] x       表示ｘ座標                                                 
//!  @param [in] y       表示ｙ座標                                                 
//!  @param [in] angle   角度                                                       
//!  @param [in] speed   速度                                                       
//!  @param [in] action  アクション番号                                             
//!  @return             なし                                                       
// -------------------------------------------------------------------------------- 
void RAMFUNC bullet00_create(int24f8_t x, int24f8_t y, uint8_t angle, int8f8_t speed, uint8_t action)
{
	ENEMYTCB	*p_tcb ;


	if( !g.enemy.remain ) {
		return ;
	}
	g.enemy.remain-- ;

	// タスク生成
	p_tcb               = task_create(NULL, bullet01, TASK_PRIO_ENEMY0) ;
	p_tcb->obj.prio     = OBJ_PRIO3 | 0 ;
	p_tcb->obj.status   = 0 ;
	p_tcb->obj.x        = x ;
	p_tcb->obj.y        = y ;
	p_tcb->obj.hit.proc = bullet00_hit ;
	p_tcb->angle        = angle ;
	p_tcb->speed        = speed ;

	obj_setAction(&p_tcb->obj, action) ;
}
