// -------------------------------------------------------------------------------- 
//!  @file  enemy00.c                                                               
//!  敵 00 タスク処理のヘッダファイル                                               
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "enemy.h"
#include "enemy00.h"


	enum {
		PARAM_POWERCAPSULE			=	1 << 15						//!< 破壊後パワーカプセル
	} ;


// -------------------------------------------------------------------------------- 
//!  敵 00 当たり処理                                                               
//!                                                                                 
//!  @param [in,out] p_self   自分                                                  
//!  @param [in,out] p_rival  相手                                                  
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static void RAMFUNC enemy00_hit(HIT *p_self, HIT *p_rival)
{
	ENEMYTCB	*p_tcb=MEMBERSTRUCT(ENEMYTCB, obj.hit, p_self) ;


	p_self->flag = 0 ;

	p_rival->proc(p_rival, p_self) ;

	g.score++ ;

	enemy_explosion(p_tcb, ACT_EXPLOSION01, (p_tcb->param&PARAM_POWERCAPSULE)!=0) ;

	sound_requestSE(3, SE_21) ;
}


// -------------------------------------------------------------------------------- 
//!  敵 00 タスク                                                                   
//!                                                                                 
//!  @param [in] p_tcb  敵タスク TCB                                                
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC enemy00( ENEMYTCB *p_tcb )
{
	p_tcb->timer0 += 16 ;

	p_tcb->obj.x -= p_tcb->speed ;
	if( p_tcb->obj.x < toFp( -16 ) ) {
		enemy_destroy( p_tcb ) ;
		return ;
	}

	p_tcb->obj.y += getSin( p_tcb->angle ) ;
	p_tcb->angle += 5 ;

	obj_updateAction( &p_tcb->obj ) ;
}


// -------------------------------------------------------------------------------- 
//!  敵 00 タスクを生成                                                             
//!                                                                                 
//!  @param [in] y      ｙ座標                                                      
//!  @param [in] param  パラメータ                                                  
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
void RAMFUNC enemy00_create(int24f8_t y, uint16_t param)
{
	ENEMYTCB	*p_tcb ;


	if( !g.enemy.remain ) {
		return ;
	}
	g.enemy.remain-- ;

	// タスク生成
	p_tcb               = task_create(NULL, enemy00, TASK_PRIO_ENEMY0) ;
	p_tcb->obj.prio     = OBJ_PRIO3 | 0 ;
	p_tcb->obj.status   = 0 ;
	p_tcb->obj.x        = toFp( SCREEN_WIDTH+8 ) ;
	p_tcb->obj.y        = y ;
	p_tcb->obj.hit.proc = enemy00_hit ;
	p_tcb->param        = param ;
	p_tcb->speed        = toFp( 1.4 ) ;
	p_tcb->angle        = 0 ;

	obj_setAction(&p_tcb->obj, ACT_ENEMY00) ;
}
