// -------------------------------------------------------------------------------- 
//!  @file  enemy01.c                                                               
//!  敵 01 タスク処理のソースファイル                                               
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "enemy.h"
#include "enemy01.h"


// -------------------------------------------------------------------------------- 
//!  弾当たり処理                                                                   
//!                                                                                 
//!  @param [in,out] p_self   自分                                                  
//!  @param [in,out] p_rival  相手                                                  
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static void RAMFUNC bullet_hit(HIT *p_self, HIT *p_rival)
{
	ENEMYTCB	*p_tcb=MEMBERSTRUCT(ENEMYTCB, obj.hit, p_self) ;


	p_self->flag = 0 ;

	p_rival->proc(p_rival, p_self) ;

	g.score++ ;

	// パワーカプセル
	if( ++g.stage.count0 == 6 ) {
		g.stage.count0 = 0 ;
	}

	enemy_explosion(p_tcb, ACT_EXPLOSION01, !g.stage.count0) ;

	sound_requestSE(3, SE_21) ;
}


// -------------------------------------------------------------------------------- 
//!  弾タスク                                                                       
//!                                                                                 
//!  @param [in] p_tcb  敵タスク TCB                                                
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC bullet( ENEMYTCB *p_tcb )
{
	// ｘ方向移動
	p_tcb->obj.x += getInt( getCos( p_tcb->angle ) * p_tcb->speed ) - toFp( g.bg.xofs ) ;
	if( (getInt( p_tcb->obj.x )<-8) || (getInt( p_tcb->obj.x )>=(SCREEN_WIDTH+8)) ) {
		enemy_destroy( p_tcb ) ;
		return ;
	}

	// ｙ方向移動
	p_tcb->obj.y += getInt( getSin( p_tcb->angle ) * p_tcb->speed ) - toFp( g.bg.yofs ) ;
	if( (getInt( p_tcb->obj.y )<-8) || (getInt( p_tcb->obj.y )>=(SCREEN_HEIGHT+8)) ) {
		enemy_destroy( p_tcb ) ;
		return ;
	}

	// 壁
	if( p_tcb->timer0 ) {
		p_tcb->timer0-- ;
	} else {
		if( bg_checkHit(p_tcb->obj.x, p_tcb->obj.y) & BG_ATTR_WALL ) {
			enemy_explosion(p_tcb, ACT_EXPLOSION00, 0) ;
			return ;
		}
	}

	obj_updateAction( &p_tcb->obj ) ;
}



// -------------------------------------------------------------------------------- 
//!  弾タスクを生成                                                                 
//!                                                                                 
//!  @param [in] x      表示ｘ座標                                                  
//!  @param [in] y      表示ｙ座標                                                  
//!  @param [in] angle  角度                                                        
//!  @param [in] speed  速度                                                        
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
void RAMFUNC createBulette(int24f8_t x, int24f8_t y, uint8_t angle, int8f8_t speed)
{
	ENEMYTCB	*p_tcb ;


	if( !g.enemy.remain ) {
		return ;
	}
	g.enemy.remain-- ;

	// タスク生成
	p_tcb               = task_create(NULL, bullet, TASK_PRIO_ENEMY0) ;
	p_tcb->obj.prio     = OBJ_PRIO3 | 0 ;
	p_tcb->obj.status   = 0 ;
	p_tcb->obj.x        = x ;
	p_tcb->obj.y        = y ;
	p_tcb->obj.hit.proc = bullet_hit ;
	p_tcb->timer0       = 16 ;
	p_tcb->angle        = angle ;
	p_tcb->speed        = speed ;

	obj_setAction(&p_tcb->obj, ACT_ENEMY01) ;
}


// -------------------------------------------------------------------------------- 
//!  敵 01 タスク                                                                   
//!                                                                                 
//!  @param [in] p_tcb  敵タスク TCB                                                
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC enemy01( ENEMYTCB *p_tcb )
{
	uint8_t		angle ;
	int8_t		range ;


	if( !g.bg.xofs ) {
		return ;
	}

	p_tcb->obj.x -= toFp( g.bg.xofs ) ;
	if( p_tcb->obj.x <= toFp( -16 ) ) {
		task_destroy( (TCB *)p_tcb ) ;
		return ;
	}

//# 位置確認
//#	obj_setPattern(&p_tcb->obj, PAT_MULTIPLE0) ;
//#

	if( --p_tcb->timer0 ) {
		return ;
	}
	p_tcb->timer0 = 120 ;

	angle = getAngle(getInt( p_tcb->obj.x ), getInt( p_tcb->obj.y ), getInt( g.player.x ), getInt( g.player.y )) ;

	range = angle - p_tcb->angle ;
	if( range < -32 ) {
		range = -32 ;
	} else
	if( range >  32 ) {
		range =  32 ;
	}

	angle = p_tcb->angle + range ;

	createBulette(p_tcb->obj.x, p_tcb->obj.y, angle-16+((getRandom()&7)-4), 256) ;
	createBulette(p_tcb->obj.x, p_tcb->obj.y, angle+ 0+((getRandom()&7)-4), 256) ;
	createBulette(p_tcb->obj.x, p_tcb->obj.y, angle+16+((getRandom()&7)-4), 256) ;
}


// -------------------------------------------------------------------------------- 
//!  敵 01 タスクを生成                                                             
//!                                                                                 
//!  @param [in] y      ｙ座標                                                      
//!  @param [in] param  パラメータ                                                  
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
void RAMFUNC enemy01_create(int24f8_t y, uint16_t param)
{
	ENEMYTCB	*p_tcb ;

	// タスク生成
	p_tcb         = task_create(NULL, enemy01, TASK_PRIO_ENEMY0) ;
	p_tcb->obj.x  = toFp( SCREEN_WIDTH ) ;
	p_tcb->obj.y  = y ;
	p_tcb->angle  = param >> 8 ;
	p_tcb->timer0 = 100 ;
}
