// -------------------------------------------------------------------------------- 
//!  @file  main.c                                                                  
//!  メイン処理のソースファイル                                                     
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"


// -------------------------------------------------------------------------------- 
//!  VSync 待ち                                                                     
//!                                                                                 
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
static inline void waitVSync( void )
{
	while( !gpio_get( VGA_CFG_PIN_VSYNC ) ) {
		;
	}

	do {
		asm volatile( "wfi" ) ;
	} while( gpio_get( VGA_CFG_PIN_VSYNC ) ) ;
}


// -------------------------------------------------------------------------------- 
//!  メイン処理                                                                     
//!                                                                                 
//!  @param [in] argc  引数の数                                                     
//!  @param [in] argv  引数                                                         
//!  @return           常に０                                                       
// -------------------------------------------------------------------------------- 
int RAMFUNC main(int argc, char *argv[])
{
	volatile uint32_t	count1 ;
	int					count3 ;


	set_sys_clock_khz(200000, true) ;

	setup_default_uart() ;

	common_init() ;

	count1 = 0 ;
	count3 = 0 ;

	while( 1 ) {
		g.vram.page ^= 1 ;

		// VSync 待ち
		waitVSync() ;

		// カウンタ更新
		count1++ ;
		g.count1 = (uint16_t )count1 ;
		g.count2 = (uint16_t )(count1 / 2) ;
		g.count4 = (uint16_t )(count1 / 4) ;
		g.count8 = (uint16_t )(count1 / 8) ;

		if( ++count3 == 3 ) {
			count3 = 0 ;
			g.count3++ ;
		}

		// パッド状態更新
		g.pad.old  = g.pad.norm ;
		g.pad.norm = g_padInfo.status ;
		g.pad.once = (g.pad.norm ^ g.pad.old) & g.pad.norm ;

#ifdef  DEBUG
		// スロー
		if( g_padInfo.status & PAD_LB ) {
			if( g.count1 & 3 ) {
				continue ;
			}
		}
#endif//DEBUG

		execVblankProc() ;

		task_exec( false ) ;

		obj_createSprite( &g.vram.spr[g.vram.page^1][0] ) ;

#ifdef  DEBUG
		// 現在の表示行（処理負荷）
		g.vram.txt[0][32] = '0' + (g_vgaInfo.dispLine / 100) ;
		g.vram.txt[0][33] = '0' + (g_vgaInfo.dispLine /  10 % 10) ;
		g.vram.txt[0][34] = '0' + (g_vgaInfo.dispLine       % 10) ;
#endif//DEBUG
	}

	return 0 ;
}
