// -------------------------------------------------------------------------------- 
//!  @file  obj.h                                                                   
//!  オブジェクト管理のヘッダファイル                                               
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#ifndef __OBJ_H__INCLUDED__
#define __OBJ_H__INCLUDED__

	//! オブジェクト状態
	enum {
		OBJ_STATUS_ACTEND_			=	0,							//!< アクション終了ビット位置
		OBJ_STATUS_SPRPRIO_			=	7,							//!< スプライト優先表示ビット位置

		OBJ_STATUS_ACTEND			=	1 << OBJ_STATUS_ACTEND_,	//!< アクション終了
		OBJ_STATUS_SPRPRIO			=	1 << OBJ_STATUS_SPRPRIO_,	//!< スプライト優先表示
	} ;

	//! オブジェクト情報
	struct _obj {
		OBJ							*p_next ;						//!< 次のオブジェクト
		uint16_t					prio ;							//!< 優先度（CC------|----PPPP）
		uint8_t						status ;						//!< 状態
		uint8_t						action ;						//!< アクション番号
		uint16_t					const *p_action ;				//!< アクションデータ
		uint16_t					pattern ;						//!< パターン番号
		uint16_t					count ;							//!< パターン表示カウンタ
		int24f8_t					x ;								//!< ｘ座標
		int24f8_t					y ;								//!< ｙ座標
		HIT							hit ;							//!< 当たり情報
	} ;


extern void RAMFUNC obj_setPattern(OBJ *p_obj, uint16_t pattern) ;
extern void RAMFUNC obj_setAction(OBJ *p_obj, uint8_t action) ;
extern void RAMFUNC obj_updateAction( OBJ *p_obj ) ;
extern void RAMFUNC obj_createSprite( VGASPR *p_spr ) ;


#endif//__OBJ_H__INCLUDED__
