// -------------------------------------------------------------------------------- 
//!  @file  powermeterselect.c                                                      
//!  パワーメーター選択画面のソースファイル                                         
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "powermeterselect.h"

#define PALPOS_SELECT				PALPOS_FIX						//!< 選択色
#define PALPOS_UNSELECT				PALPOS_SELGUAGE					//!< 未選択色
#define PALPOS_RECT					PALPOS_POWERMETERSELECT			//!< 矩形色
#define RECTPOS						(CHRPOS_POWERMETERSELECT+208)	//!< 矩形パターン位置
#define FILLPOS						0x23							//!< 塗り潰しパターン位置

	//! 矩形
	typedef struct {
		uint8_t						x ;								//!< ｘ座標
		uint8_t						y ;								//!< ｙ座標
		uint8_t						w ;								//!< 幅
		uint8_t						h ;								//!< 高さ
	} RECT ;

	//! レイアウト
	typedef struct {
		uint8_t						sheildY ;						//!< シールドｙ座標
		uint8_t						multipleY ;						//!< マルチプルｙ座標
		RECT						sheildRect[2] ;					//!< シールド矩形
		RECT						multipleRect[3] ;				//!< マルチプル矩形
	} LAYOUT ;

	// システム子タスク TCB
	typedef struct {
		TCBHEAD( SYSTEMTCB )										//!< TCB ヘッダ
		OBJ							obj ;							//!< オブジェクト
		LAYOUT						const *p_layout ;				//!< レイアウト
		uint8_t						id ;							//!< 識別子
		uint8_t						angle ;							//!< 角度
		uint16_t					color ;							//!< 色
		uint16_t					*p_palette ;					//!< パレットアドレス
		RECT						rect ;							//!< 矩形
		uint8_t						palette ;						//!< パレット番号
		uint8_t						fill ;							//!< 塗り潰しフラグ
	} SYSCHILDTCB ;


static void selectPlayerType( SYSTEMTCB *p_tcb ) ;
static void selectPlayerType_040( SYSTEMTCB *p_tcb ) ;
static void selectPlayerType_080( SYSTEMTCB *p_tcb ) ;
static void selectShield( SYSTEMTCB *p_tcb ) ;
static void selectShield_040( SYSTEMTCB *p_tcb ) ;
static void selectShield_080( SYSTEMTCB *p_tcb ) ;
static void selectMultiple( SYSTEMTCB *p_tcb ) ;
static void selectMultiple_040( SYSTEMTCB *p_tcb ) ;
static void selectMultiple_080( SYSTEMTCB *p_tcb ) ;
static void selectMultiple_0c0( SYSTEMTCB *p_tcb ) ;
static void selectMultiple_100( SYSTEMTCB *p_tcb ) ;


static const uint16_t				m_playerTypeTile[][14*6]={		//!< プレイヤー種類タイルデータ
										{		// タイプ１
											0x0000,0x8106,0x8107,0x8108,0x8109,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
											0x0000,0x810a,0x810b,0x0000,0x810c,0x810d,0x810e,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
											0x810f,0x8110,0x8111,0x8112,0x8113,0x8114,0x8115,0x8116,0x8117,0x8118,0x8119,0x0000,0x0000,0x0000,
											0x811a,0x811b,0x811c,0x811d,0x811e,0x811f,0x8120,0x8121,0x8122,0x8123,0x8124,0x8125,0x8126,0x0000,
											0x8127,0x8128,0x8129,0x812a,0x812b,0x812c,0x812d,0x812e,0x812f,0x8130,0x8131,0x8132,0x8133,0x8134,
											0x8135,0x8136,0x8137,0x8138,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000
										}, {	// タイプ２
											0x0000,0x9139,0x913a,0x913b,0x913c,0x913d,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
											0x913e,0x913f,0x9140,0x9141,0x9142,0x9143,0x9144,0x9145,0x9146,0x9147,0x9148,0x9149,0x914a,0x914b,
											0x914c,0x914d,0x914e,0x914f,0x9150,0x9151,0x9152,0x9153,0x9154,0x9155,0x9156,0x9157,0x9158,0x9159,
											0x0000,0x0000,0x0000,0x915a,0x915b,0x915c,0x915d,0x915e,0x915f,0x0000,0x0000,0x0000,0x0000,0x0000,
											0x0000,0x0000,0x0000,0x9160,0x9161,0x9162,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
											0x0000,0x0000,0x0000,0x9163,0x9164,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000
										}, {	// タイプ３
											0x0000,0xa165,0xa166,0xa167,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
											0x0000,0xa168,0xa169,0xa16a,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
											0xa16b,0xa16c,0xa16d,0xa16e,0xa16f,0xa170,0xa171,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
											0xa172,0xa173,0xa174,0xa175,0xa176,0xa177,0xa178,0xa179,0xa17a,0xa17b,0xa17c,0xa17d,0xa17e,0xa17f,
											0xa180,0xa181,0xa182,0xa183,0xa184,0xa185,0xa186,0xa187,0xa188,0xa189,0xa18a,0xa18b,0xa18c,0xa18d,
											0xa18e,0xa18f,0xa190,0xa191,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000
										}, {	// タイプ４
											0xb192,0xb193,0xb194,0xb195,0xb196,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
											0xb197,0xb198,0xb199,0xb19a,0xb19b,0xb19c,0xb19d,0xb19e,0xb19f,0x0000,0x0000,0x0000,0x0000,0x0000,
											0x0000,0xb1a0,0xb1a1,0xb1a2,0xb1a3,0xb1a4,0xb1a5,0xb1a6,0xb1a7,0xb1a8,0xb1a9,0xb1aa,0xb1ab,0x0000,
											0x0000,0xb1ac,0xb1ad,0xb1ae,0xb1af,0xb1b0,0xb1b1,0xb1b2,0xb1b3,0xb1b4,0xb1b5,0xb1b6,0xb1b7,0xb1b8,
											0xb1b9,0xb1ba,0xb1bb,0xb1bc,0xb1bd,0xb1be,0xb1bf,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
											0xb1c0,0xb1c1,0xb1c2,0xb1c3,0xb1c4,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000
										}
									} ;

static uint16_t						m_rectTile[][9][6]={			//!<  矩形タイルデータ
										{		// 閉じる
											{ RECTPOS+0x0001,RECTPOS+0x0801,RECTPOS+0x0401,RECTPOS+0x0c01,RECTPOS+0x000a,RECTPOS+0x000c },
											{ RECTPOS+0x0001,RECTPOS+0x0801,RECTPOS+0x0006,RECTPOS+0x0806,RECTPOS+0x000a,RECTPOS+0x000d },
											{ RECTPOS+0x0001,RECTPOS+0x0801,RECTPOS+0x0007,RECTPOS+0x0807,RECTPOS+0x000a,RECTPOS+0x000e },
											{ RECTPOS+0x0001,RECTPOS+0x0801,RECTPOS+0x0008,RECTPOS+0x0808,RECTPOS+0x000a,RECTPOS+0x000f },
											{ RECTPOS+0x0002,RECTPOS+0x0802,RECTPOS+0x0009,RECTPOS+0x0809,RECTPOS+0x000b,RECTPOS+0x0010 },
											{ RECTPOS+0x0003,RECTPOS+0x0803,RECTPOS+0x0000,RECTPOS+0x0000,RECTPOS+0x0000,RECTPOS+0x0003 },
											{ RECTPOS+0x0004,RECTPOS+0x0804,RECTPOS+0x0000,RECTPOS+0x0000,RECTPOS+0x0000,RECTPOS+0x0004 },
											{ RECTPOS+0x0005,RECTPOS+0x0805,RECTPOS+0x0000,RECTPOS+0x0000,RECTPOS+0x0000,RECTPOS+0x0005 },
											{ RECTPOS+0x0000,RECTPOS+0x0000,RECTPOS+0x0000,RECTPOS+0x0000,RECTPOS+0x0000,RECTPOS+0x0000 },
										}, {	// 開く
											{ RECTPOS+0x0000,RECTPOS+0x0000,RECTPOS+0x0000,RECTPOS+0x0000,RECTPOS+0x0000,RECTPOS+0x0000 },
											{ RECTPOS+0x0000,RECTPOS+0x0000,RECTPOS+0x0405,RECTPOS+0x0c05,RECTPOS+0x0000,RECTPOS+0x0405 },
											{ RECTPOS+0x0000,RECTPOS+0x0000,RECTPOS+0x0404,RECTPOS+0x0c04,RECTPOS+0x0000,RECTPOS+0x0404 },
											{ RECTPOS+0x0000,RECTPOS+0x0000,RECTPOS+0x0403,RECTPOS+0x0c03,RECTPOS+0x0000,RECTPOS+0x0403 },
											{ RECTPOS+0x0409,RECTPOS+0x0c09,RECTPOS+0x0402,RECTPOS+0x0c02,RECTPOS+0x040b,RECTPOS+0x0410 },
											{ RECTPOS+0x0408,RECTPOS+0x0c08,RECTPOS+0x0401,RECTPOS+0x0c01,RECTPOS+0x040a,RECTPOS+0x040f },
											{ RECTPOS+0x0407,RECTPOS+0x0c07,RECTPOS+0x0401,RECTPOS+0x0c01,RECTPOS+0x040a,RECTPOS+0x040e },
											{ RECTPOS+0x0406,RECTPOS+0x0c06,RECTPOS+0x0401,RECTPOS+0x0c01,RECTPOS+0x040a,RECTPOS+0x040d },
											{ RECTPOS+0x0001,RECTPOS+0x0801,RECTPOS+0x0401,RECTPOS+0x0c01,RECTPOS+0x040a,RECTPOS+0x040c },
										}
									} ;

static const RECT					m_playerTypeRect[]={			//!< プレイヤー種類矩形
										{  4,  3, 16, 8 },			//!< タイプ１
										{ 20,  3, 16, 8 },			//!< タイプ２
										{  4, 19, 16, 8 },			//!< タイプ３
										{ 20, 19, 16, 8 },			//!< タイプ４
									} ;

static const RECT					m_weaponRect={					//!< 武器矩形
										4, 11, 32, 8
									} ;

static const LAYOUT					m_layout[]={					//!< レイアウト
										{
											184,
											56,
											{
												{  7, 20, 10, 6 },
												{ 23, 20, 10, 6 }
											}, {
												{  5,  3, 10, 8 },
												{ 15,  3, 10, 8 },
												{ 25,  3, 10, 8 }
											}
										}, {
											56,
											184,
											{
												{  7,  4, 10, 6 },
												{ 23,  4, 10, 6 }
											}, {
												{  5, 19, 10, 8 },
												{ 15, 19, 10, 8 },
												{ 25, 19, 10, 8 }
											}
										}
									} ;


// -------------------------------------------------------------------------------- 
//!  タイル面へタイルデータを描画                                                   
//!                                                                                 
//!  @param [in] x       ｘ座標                                                     
//!  @param [in] y       ｙ座標                                                     
//!  @param [in] w       幅                                                         
//!  @param [in] h       高さ                                                       
//!  @param [in] p_tile  タイルデータ                                               
//!  @return             なし                                                       
// -------------------------------------------------------------------------------- 
static void drawTile(int x, int y, int w, int h, uint16_t const *p_tile)
{
	uint16_t	*p_dest, *p_destEnd, *p_lineEnd ;
	int			add ;


	p_dest    = &g.vram.tile[1][y][x] ;
	p_destEnd = &g.vram.tile[1][y+h][x] ;
	add       = VGA_TILE_WIDTH_COUNT - w ;

	do {
		p_lineEnd = &p_dest[w] ;
		do {
			*p_dest = *p_tile++ ;
		} while( ++p_dest != p_lineEnd ) ;

		p_dest += add ;
	} while( p_dest != p_destEnd ) ;
}


// -------------------------------------------------------------------------------- 
//!  タイル面へ矩形枠を描画                                                         
//!                                                                                 
//!  @param [in] p_rect   矩形                                                      
//!  @param [in] palette  パレット番号                                              
//!  @param [in] pattern  パターン番号                                              
//!  @param [in] open     - true :開く                                              
//!                       - false:閉じる                                            
//!  @return              なし                                                      
// -------------------------------------------------------------------------------- 
static void drawRect(RECT const *p_rect, int palette, int pattern, bool open)
{
	uint16_t	tile, *p_destLU, *p_destRD, *p_destEnd ;
	uint16_t	const *p_tile ;


	p_tile = &m_rectTile[open][pattern][0] ;
	tile   = palette << VGATILE_PC_BIT ;

	// 左上
	p_destLU    = &g.vram.tile[1][p_rect->y][p_rect->x] ;
	*p_destLU++ = p_tile[0] | tile ;

	// 左下
	p_destRD    = &g.vram.tile[1][p_rect->y+p_rect->h-1][p_rect->x] ;
	*p_destRD++ = p_tile[2] | tile ;

	// 水平
	p_destEnd = &p_destLU[p_rect->w-2] ;
	while( p_destLU != p_destEnd ) {
		*p_destLU++ = p_tile[4] | tile ;
		*p_destRD++ = p_tile[4] | tile ;
	}

	// 右上
	*p_destLU++ = p_tile[1] | tile ;

	// 右下
	*p_destRD++ = p_tile[3] | tile ;

	// 垂直
	p_destLU  = &g.vram.tile[1][p_rect->y+1][p_rect->x] ;
	p_destRD  = &g.vram.tile[1][p_rect->y+1][p_rect->x+p_rect->w-1] ;
	p_destEnd = &g.vram.tile[1][p_rect->y+p_rect->h-1][p_rect->x] ;

	while( p_destLU != p_destEnd ) {
		*p_destLU = p_tile[5] | tile ; ;
		p_destLU += VGA_TILE_WIDTH_COUNT ;

		*p_destRD = p_tile[5] | tile ; ;
		p_destRD += VGA_TILE_WIDTH_COUNT ;
	}
}


// -------------------------------------------------------------------------------- 
//!  タイル面を指定タイルデータで矩形塗り潰し                                       
//!                                                                                 
//!  @param [in] p_rect   矩形                                                      
//!  @param [in] palette  パレット番号                                              
//!  @param [in] pattern  パターン番号                                              
//!  @param [in] open     - true :開く                                              
//!                       - false:閉じる                                            
//!  @return              なし                                                      
// -------------------------------------------------------------------------------- 
static void fillRect(RECT const *p_rect, int palette, int pattern, bool open)
{
	uint16_t	tile, *p_tile, *p_tileEnd, *p_dest, *p_destEnd ;


	p_tile    = &g.vram.tile[1][p_rect->y][p_rect->x] ;
	p_tileEnd = &g.vram.tile[1][p_rect->y+p_rect->h][p_rect->x] ;

	if( open != false ) {
		tile = (palette << VGATILE_PC_BIT) | VGATILE_VF | (FILLPOS + 8 - pattern) ;
	} else {
		tile = (palette << VGATILE_PC_BIT) |              (FILLPOS +     pattern) ;
	}

	do {
		p_dest    = &p_tile[0] ;
		p_destEnd = &p_tile[p_rect->w] ;

		do {
			*p_dest = tile ;
		} while( ++p_dest != p_destEnd ) ;

		p_tile += VGA_TILE_WIDTH_COUNT ;
	} while( p_tile != p_tileEnd ) ;
}


// -------------------------------------------------------------------------------- 
//!  カーソル移動                                                                   
//!                                                                                 
//!  @param [in] p_unselect  未選択矩形                                             
//!  @param [in] p_select    選択矩形                                               
//!  @return                 なし                                                   
// -------------------------------------------------------------------------------- 
static void moveCursor(RECT const *p_unselect, RECT const *p_select)
{
	drawRect(p_unselect, PALPOS_UNSELECT, 0, false) ;
	drawRect(p_select,   PALPOS_SELECT,   0, false) ;
}


// -------------------------------------------------------------------------------- 
//!  パターン表示タスクを作成                                                       
//!                                                                                 
//!  @param [in] p_parent  親タスク TCB                                             
//!  @param [in] taskProc  タスク処理                                               
//!  @param [in] prio      優先順位                                                 
//!  @param [in] x         ｘ座標                                                   
//!  @param [in] y         ｙ座標                                                   
//!  @param [in] pattern   パターン番号                                             
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
static SYSCHILDTCB *createPatternTask(SYSTEMTCB *p_parent, TASKPROC taskProc, uint8_t prio, int32_t x, int32_t y, uint16_t pattern)
{
	SYSCHILDTCB		*p_tcb ;


	p_tcb              = task_create((TCB *)p_parent, taskProc, prio) ;
	p_tcb->p_parent    = p_parent ;
	p_tcb->obj.prio    = OBJ_PRIO0 ;
	p_tcb->obj.status  = OBJ_STATUS_SPRPRIO ;
	p_tcb->obj.pattern = pattern ;
	p_tcb->obj.x       = toFp( x ) ;
	p_tcb->obj.y       = toFp( y ) ;

	return p_tcb ;
}


// -------------------------------------------------------------------------------- 
//!  アクションタスクを作成                                                         
//!                                                                                 
//!  @param [in] p_parent  親タスク TCB                                             
//!  @param [in] taskProc  タスク処理                                               
//!  @param [in] prio      優先順位                                                 
//!  @param [in] x         ｘ座標                                                   
//!  @param [in] y         ｙ座標                                                   
//!  @param [in] action    アクション番号                                           
//!  @param [in] id        識別子                                                   
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
static SYSCHILDTCB *createActionTask(SYSTEMTCB *p_parent, TASKPROC taskProc, uint8_t prio, int32_t x, int32_t y, uint8_t action, uint8_t id)
{
	SYSCHILDTCB		*p_tcb ;


	p_tcb             = task_create((TCB *)p_parent,taskProc, prio) ;
	p_tcb->p_parent   = p_parent ;
	p_tcb->obj.prio   = OBJ_PRIO3 ;
	p_tcb->obj.status = OBJ_STATUS_SPRPRIO ;
	p_tcb->obj.x      = toFp( x ) ;
	p_tcb->obj.y      = toFp( y ) ;
	p_tcb->id         = id ;

	obj_setAction(&p_tcb->obj, action) ;

	return p_tcb ;
}


// -------------------------------------------------------------------------------- 
//!  パレット更新タスク処理                                                         
//!                                                                                 
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
static void updatePalette( TCB *p_tcb )
{
	uint16_t	*p_srce ;
	int			color ;

	//
	p_srce                      = &g.vram.pal[PALPOS_VIXENB][(g.count2&1)*4] ;
	g.vram.pal[PALPOS_VIXEN][1] = p_srce[1] ;
	g.vram.pal[PALPOS_VIXEN][2] = p_srce[2] ;
	g.vram.pal[PALPOS_VIXEN][3] = p_srce[3] ;

	//
	if( g.powerMeter.playerType == 2 ) {
		color = getInt( getSin( g.count1 & 127 ) * 17 ) ;
		if( color > 15 ) {
			color = 15 ;
		}

		g.vram.pal[PALPOS_VIXEN][6] = color ;
	}
}


// -------------------------------------------------------------------------------- 
//!  パターン表示タスク処理                                                         
//!                                                                                 
//!  @param [in,out] p_tcb  TCB                                                     
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void patternTask( SYSCHILDTCB *p_tcb )
{
	obj_setPattern(&p_tcb->obj, p_tcb->obj.pattern) ;
}


// -------------------------------------------------------------------------------- 
//!  装備タスク処理                                                                 
//!                                                                                 
//!  @param [in,out] p_tcb  TCB                                                     
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void weaponTask( SYSCHILDTCB *p_tcb )
{
	obj_setPattern(&p_tcb->obj, PAT_WEAPON_TYPE1+g.powerMeter.playerType) ;
}


// -------------------------------------------------------------------------------- 
//!  シールドタスク処理                                                             
//!                                                                                 
//!  @param [in,out] p_tcb  TCB                                                     
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void shieldTask( SYSCHILDTCB *p_tcb )
{
	if( p_tcb->id == g.powerMeter.shield ) {
		obj_updateAction( &p_tcb->obj ) ;
	} else {
		obj_setPattern(&p_tcb->obj, p_tcb->obj.pattern) ;
	}
}


// -------------------------------------------------------------------------------- 
//!  マルチプルタスク処理                                                           
//!                                                                                 
//!  @param [in,out] p_tcb  TCB                                                     
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void multipleTask( SYSCHILDTCB *p_tcb )
{
	if( p_tcb->id == g.powerMeter.multiple ) {
		obj_updateAction( &p_tcb->obj ) ;
	} else {
		obj_setPattern(&p_tcb->obj, p_tcb->obj.pattern) ;
	}
}


// -------------------------------------------------------------------------------- 
//!  ローリングマルチプルタスク処理                                                 
//!                                                                                 
//!  @param [in,out] p_tcb  TCB                                                     
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void rollingMultipleTask( SYSCHILDTCB *p_tcb )
{
	if( p_tcb->id == g.powerMeter.multiple ) {
		p_tcb->angle += 5 ;
		p_tcb->obj.x  = toFp( 240 )                        + (getCos( p_tcb->angle ) * 24) ;
		p_tcb->obj.y  = toFp( p_tcb->p_layout->multipleY ) + (getSin( p_tcb->angle ) * 24) ;
		obj_updateAction( &p_tcb->obj ) ;
	} else {
		obj_setPattern(&p_tcb->obj, p_tcb->obj.pattern) ;
	}
}


// -------------------------------------------------------------------------------- 
//!  落ちるパレット処理                                                             
//!                                                                                 
//!  @param [in,out] p_tcb  TCB                                                     
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void fallPalette( SYSCHILDTCB *p_tcb )
{
	if( !(p_tcb->timer0&3) ) {
		*p_tcb->p_palette++ = p_tcb->color ;
	}

	if( ++p_tcb->timer0 == (8*4) ) {
		p_tcb->p_parent->childCount-- ;
		task_destroy( (TCB *)p_tcb ) ;
		return ;
	}
}


// -------------------------------------------------------------------------------- 
//!  落ちるパレットタスク                                                           
//!                                                                                 
//!  @param [in,out] p_parent  親タスク TCB                                         
//!  @param [in]     palette   パレット番号                                         
//!  @param [in]     color     色                                                   
//!  @return                   なし                                                 
// -------------------------------------------------------------------------------- 
static void createFallPaletteTask(SYSTEMTCB *p_parent, int palette, uint16_t color)
{
	SYSCHILDTCB		*p_tcb ;


	p_tcb            = task_create((TCB *)p_parent,fallPalette, TASK_PRIO_SYSTEM+1) ;
	p_tcb->p_parent  = p_parent ;
	p_tcb->timer0    = 0 ;
	p_tcb->p_palette = &g.vram.pal[palette][4] ;
	p_tcb->color     = color ;

	p_parent->childCount++ ;
}


// -------------------------------------------------------------------------------- 
//!  矩形を開く処理                                                                 
//!                                                                                 
//!  @param [in,out] p_tcb  TCB                                                     
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void openRect( SYSCHILDTCB *p_tcb )
{
	RECT	rect ;
	int		index ;


	if( !(p_tcb->timer0&3) ) {
		index = p_tcb->timer0 / 4 ;

		if( p_tcb->fill ) {
			rect.x = p_tcb->rect.x + 1 ;
			rect.y = p_tcb->rect.y + 1 ;
			rect.w = p_tcb->rect.w - 2 ;
			rect.h = p_tcb->rect.h - 2 ;
			fillRect(&rect, 0, 1+index, true) ;
		}

		drawRect(&p_tcb->rect, p_tcb->palette, 1+index, true) ;
	}

	if( ++p_tcb->timer0 == (8*4) ) {
		p_tcb->p_parent->childCount-- ;
		task_destroy( (TCB *)p_tcb ) ;
		return ;
	}
}


// -------------------------------------------------------------------------------- 
//!  矩形を開くタスクを作成                                                         
//!                                                                                 
//!  @param [in,out] p_parent  親タスク TCB                                         
//!  @param [in]     p_rect    矩形                                                 
//!  @param [in]     palette   パレット番号                                         
//!  @param [in]     fill      塗り潰し                                             
//!                            - true :あり                                         
//!                            - false:なし                                         
//!  @return                   なし                                                 
// -------------------------------------------------------------------------------- 
static void createOpenRectTask(SYSTEMTCB *p_parent, RECT const *p_rect, int palette, bool fill)
{
	SYSCHILDTCB		*p_tcb ;


	p_tcb           = task_create((TCB *)p_parent,openRect, TASK_PRIO_SYSTEM+1) ;
	p_tcb->p_parent = p_parent ;
	p_tcb->timer0   = 0 ;
	p_tcb->rect     = *p_rect ;
	p_tcb->palette  = palette ;
	p_tcb->fill     = fill ;

	p_parent->childCount++ ;
}


// -------------------------------------------------------------------------------- 
//!  矩形を閉じる処理                                                               
//!                                                                                 
//!  @param [in,out] p_tcb  TCB                                                     
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void closeRect( SYSCHILDTCB *p_tcb )
{
	RECT	rect ;
	int		index ;


	if( !(p_tcb->timer0&3) ) {
		index  = p_tcb->timer0 / 4 ;

		if( index == 7 ) {	// 全消去
			fillRect(&p_tcb->rect, 0, 8, false) ;
		} else {			// 部分消去
			if( p_tcb->fill ) {
				rect.x = p_tcb->rect.x + 1 ;
				rect.y = p_tcb->rect.y + 1 ;
				rect.w = p_tcb->rect.w - 2 ;
				rect.h = p_tcb->rect.h - 2 ;
				fillRect(&rect, 0, 1+index, false) ;
			}

			drawRect(&p_tcb->rect, p_tcb->palette, 1+index, false) ;
		}
	}

	if( ++p_tcb->timer0 == (8*4) ) {
		p_tcb->p_parent->childCount-- ;
		task_destroy( (TCB *)p_tcb ) ;
		return ;
	}
}


// -------------------------------------------------------------------------------- 
//!  矩形を閉じるタスクを作成                                                       
//!                                                                                 
//!  @param [in,out] p_parent  親タスク TCB                                         
//!  @param [in]     p_rect    矩形                                                 
//!  @param [in]     palette   パレット番号                                         
//!  @param [in]     fill      塗り潰し                                             
//!                            - true :あり                                         
//!                            - false:なし                                         
//!  @return                   なし                                                 
// -------------------------------------------------------------------------------- 
static void createCloseRectTask(SYSTEMTCB *p_parent, RECT const *p_rect, int palette, bool fill)
{
	SYSCHILDTCB		*p_tcb ;


	p_tcb           = task_create((TCB *)p_parent,closeRect, TASK_PRIO_SYSTEM+1) ;
	p_tcb->p_parent = p_parent ;
	p_tcb->timer0   = 0 ;
	p_tcb->rect     = *p_rect ;
	p_tcb->palette  = palette ;
	p_tcb->fill     = fill ;

	p_parent->childCount++ ;
}


// -------------------------------------------------------------------------------- 
//!  カーソル点滅タスク処理                                                         
//!                                                                                 
//!  @param [in,out] p_tcb  タスクコントロールブロック                              
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void blinkCursor( SYSCHILDTCB *p_tcb )
{
	int		index, palette ;


	if( !(p_tcb->timer0&3) ) {
		index = p_tcb->timer0 / 4 ;

		palette = PALPOS_UNSELECT ;
		if( index & 1 ) {
			palette = PALPOS_SELECT ;
			if( index == 15 ) {
				palette = PALPOS_RECT ;
			}
		}

		drawRect(&p_tcb->rect, palette, 0, false) ;
	}

	if( ++p_tcb->timer0 == 64 ) {
		p_tcb->p_parent->childCount-- ;
		task_destroy( (TCB *)p_tcb ) ;
		return ;
	}
}


// -------------------------------------------------------------------------------- 
//!  カーソル点滅タスクを作成                                                       
//!                                                                                 
//!  @param [in,out] p_parent  親タスク TCB                                         
//!  @param [in]     p_rect    矩形                                                 
//!  @return                   なし                                                 
// -------------------------------------------------------------------------------- 
static void createBlinkCursorTask(SYSTEMTCB *p_parent, RECT const *p_rect)
{
	SYSCHILDTCB		*p_tcb ;


	p_tcb           = task_create((TCB *)p_parent,blinkCursor, TASK_PRIO_PLAYER) ;
	p_tcb->p_parent = p_parent ;
	p_tcb->timer0   = 0 ;
	p_tcb->rect     = *p_rect ;

	p_parent->childCount++ ;
}


// -------------------------------------------------------------------------------- 
//!  パワーメーター選択 VBlank 処理                                                 
//!                                                                                 
//!  @param [in,out] p_tcb  システムタスク TCB                                      
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void RAMFUNC powermeterselect_vblank( SYSTEMTCB *p_tcb )
{
	vga_setFgTile(&g.vram.tile[1][0][0],       &g.vram.pat[0][0], &g.vram.pal[0][0]) ;
	vga_setSprite(&g.vram.spr[g.vram.page][0], &g.vram.pat[0][0], &g.vram.pal[0][0]) ;
	vga_setTextOffset(0, 0) ;
}


// -------------------------------------------------------------------------------- 
//!  パワーメーター選択                                                             
//!                                                                                 
//!  @param [in,out] p_tcb  システムタスク TCB                                      
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
void powermeterselect( SYSTEMTCB *p_tcb )
{
	DBGASSERT( sizeof( SYSCHILDTCB ) <= sizeof( TCB ) )

	// 非表示
	vga_setFgBlank() ;
	vga_setSpriteBlank() ;
	vga_setTextBlank() ;

	// 初期化
	setMemory(&g.vram.tile[0][0][0], 0, sizeof( g.vram.tile )) ;
	setMemory(&g.vram.pal[0][0],     0, sizeof( g.vram.pal )) ;
	g.powerMeter.playerType = 0 ;
	g.powerMeter.multiple   = 0 ;
	g.powerMeter.shield     = 0 ;
	p_tcb->childCount       = 0 ;

	// データ転送
	xferData(XFERDATA_FIX_OFFSET,              XFERDATA_FIX_COUNT) ;
	xferData(XFERDATA_POWERMETERSELECT_OFFSET, XFERDATA_POWERMETERSELECT_COUNT) ;

	// 文字列
	writeTileString16(2, 1, "POWER METER SELECT", 0) ;

	// プレイヤータイプ描画
	drawTile( 5,  4, 14, 6, &m_playerTypeTile[0][0]) ;
	drawTile(21,  4, 14, 6, &m_playerTypeTile[1][0]) ;
	drawTile( 5, 20, 14, 6, &m_playerTypeTile[2][0]) ;
	drawTile(21, 20, 14, 6, &m_playerTypeTile[3][0]) ;

	// 装備
	fillRect(&m_weaponRect, 0, 0, true) ;
	createPatternTask(p_tcb, patternTask, TASK_PRIO_WEAPON,  44, 104, PAT_WEAPON_TYPEL) ;
	createPatternTask(p_tcb, patternTask, TASK_PRIO_WEAPON, 204, 104, PAT_WEAPON_TYPER) ;
	createPatternTask(p_tcb, weaponTask,  TASK_PRIO_WEAPON,  84, 104, PAT_WEAPON_TYPE1) ;

	// オープン
	createFallPaletteTask(p_tcb,  8, g.vram.pal[PALPOS_POWERMETERSELECTB][0]) ;
	createFallPaletteTask(p_tcb,  9, 0x0888) ;
	createFallPaletteTask(p_tcb, 10, 0x0888) ;
	createFallPaletteTask(p_tcb, 11, 0x0888) ;
	createOpenRectTask(p_tcb, &m_playerTypeRect[0], PALPOS_SELECT,   false) ;
	createOpenRectTask(p_tcb, &m_playerTypeRect[1], PALPOS_UNSELECT, false) ;
	createOpenRectTask(p_tcb, &m_playerTypeRect[2], PALPOS_UNSELECT, false) ;
	createOpenRectTask(p_tcb, &m_playerTypeRect[3], PALPOS_UNSELECT, false) ;
	createOpenRectTask(p_tcb, &m_weaponRect,        PALPOS_RECT,     true) ;

	setVSyncProc(powermeterselect_vblank, p_tcb) ;

	p_tcb->taskProc = selectPlayerType ;

	sound_playBGM( BGM_01 ) ;
}


// -------------------------------------------------------------------------------- 
//!  プレイヤー選択                                                                 
//!                                                                                 
//!  @param [in,out] p_tcb  システムタスク TCB                                      
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
void selectPlayerType( SYSTEMTCB *p_tcb )
{
	static const int8_t		playerType[][4]={
								{ 2, 2, 3, 1 },		// 0
								{ 3, 3, 0, 2 },		// 1
								{ 0, 0, 1, 3 },		// 2
								{ 1, 1, 2, 0 },		// 3
							} ;
	uint8_t					playerTypeOld ;


	if( p_tcb->childCount ) {
		return ;
	}

	// 選択
	if( g.pad.once & (PAD_BTN( 0 )|PAD_BTN( 1 )) ) {
		createBlinkCursorTask(p_tcb, &m_playerTypeRect[g.powerMeter.playerType]) ;

		p_tcb->taskProc = selectPlayerType_040 ;

		sound_requestSE(1, SE_02) ;
		return ;
	}

	// 移動なしなら無視
	if( !(g.pad.once&(PAD_UP|PAD_DOWN|PAD_LEFT|PAD_RIGHT)) ) {
		return ;
	}

	playerTypeOld = g.powerMeter.playerType ;

	if( g.pad.once & (PAD_UP|PAD_DOWN) ) {
		g.powerMeter.playerType = playerType[g.powerMeter.playerType][   (g.pad.once&PAD_DOWN) !=0] ;
	} else {
		g.powerMeter.playerType = playerType[g.powerMeter.playerType][2+((g.pad.once&PAD_RIGHT)!=0)] ;
	}

	if( g.powerMeter.playerType == playerTypeOld ) {
		return ;
	}

	// カーソル移動
	moveCursor(&m_playerTypeRect[playerTypeOld], &m_playerTypeRect[g.powerMeter.playerType]) ;
	createFallPaletteTask(p_tcb, 8+g.powerMeter.playerType, g.vram.pal[PALPOS_POWERMETERSELECTB][g.powerMeter.playerType]) ;
	createFallPaletteTask(p_tcb, 8+playerTypeOld,           0x0888) ;

	sound_requestSE(0, SE_00) ;
}


static void selectPlayerType_040( SYSTEMTCB *p_tcb )
{
	int		index ;


	if( p_tcb->childCount ) {
		return ;
	}

	p_tcb->taskProc = selectPlayerType_080 ;

	for(index=0; index<4; index++) {
		if( index != g.powerMeter.playerType ) {
			createFallPaletteTask(p_tcb, 8+index, 0x0000) ;
			createCloseRectTask(p_tcb, &m_playerTypeRect[index], PALPOS_UNSELECT, false) ;
		}
	}
}


static void selectPlayerType_080( SYSTEMTCB *p_tcb )
{
	LAYOUT	const *p_layout ;


	if( p_tcb->childCount ) {
		return ;
	}

	// ヴィクセンデータ転送
	xferData(g_xferVixen[g.powerMeter.playerType][0], g_xferVixen[g.powerMeter.playerType][1]) ;

	// パレット更新タスク
	task_create((TCB *)p_tcb, updatePalette, TASK_PRIO_SYSTEM+1) ;

	p_layout = &m_layout[g.powerMeter.playerType/2] ;

	// バリア
	createPatternTask(p_tcb, patternTask, TASK_PRIO_SHIELD+0,  88, p_layout->sheildY,   PAT_VIXEN2) ;
	createActionTask(p_tcb,  shieldTask,  TASK_PRIO_SHIELD+0, 112, p_layout->sheildY-8, ACT_SHIELDU0,    0) ;
	createActionTask(p_tcb,  shieldTask,  TASK_PRIO_SHIELD+0, 112, p_layout->sheildY+8, ACT_SHIELDD0,    0) ;

	// フォースフィールド
	createPatternTask(p_tcb, patternTask, TASK_PRIO_SHIELD+1, 224, p_layout->sheildY,   PAT_VIXEN2) ;
	createActionTask(p_tcb,  shieldTask,  TASK_PRIO_SHIELD+1, 224, p_layout->sheildY,   ACT_FORCEFIELD1, 1) ;

	createOpenRectTask(p_tcb, &p_layout->sheildRect[0], PALPOS_SELECT,   true) ;
	createOpenRectTask(p_tcb, &p_layout->sheildRect[1], PALPOS_UNSELECT, true) ;

	p_tcb->taskProc = selectShield ;
}


// -------------------------------------------------------------------------------- 
//!  シールド選択                                                                   
//!                                                                                 
//!  @param [in,out] p_tcb  システムタスク TCB                                      
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void selectShield( SYSTEMTCB *p_tcb )
{
	LAYOUT	const *p_layout ;


	if( p_tcb->childCount ) {
		return ;
	}

	if( g.pad.once & (PAD_BTN( 0 )|PAD_BTN( 1 )) ) {
		p_layout = &m_layout[g.powerMeter.playerType/2] ;
		createBlinkCursorTask(p_tcb, &p_layout->sheildRect[g.powerMeter.shield]) ;

		p_tcb->taskProc = selectShield_040 ;

		sound_requestSE(1, SE_02) ;
		return ;
	}

	if( (!g.powerMeter.shield&&(g.pad.once&PAD_RIGHT)) || ((g.powerMeter.shield==1)&&(g.pad.once&PAD_LEFT)) ) {
		g.powerMeter.shield ^= 1 ;

		p_layout = &m_layout[g.powerMeter.playerType/2] ;
		moveCursor(&p_layout->sheildRect[g.powerMeter.shield^1], &p_layout->sheildRect[g.powerMeter.shield]) ;

		sound_requestSE(0, SE_00) ;
	}
}


static void selectShield_040( SYSTEMTCB *p_tcb )
{
	LAYOUT	const *p_layout ;


	if( p_tcb->childCount ) {
		return ;
	}

	// プレーヤータイプ消去
	createFallPaletteTask(p_tcb, 8+g.powerMeter.playerType, 0x0000) ;
	createCloseRectTask(p_tcb, &m_playerTypeRect[g.powerMeter.playerType], PALPOS_RECT, false) ;

	// シールド消去
	p_layout = &m_layout[g.powerMeter.playerType/2] ;
	createCloseRectTask(p_tcb, &p_layout->sheildRect[g.powerMeter.shield^1], PALPOS_UNSELECT, true) ;

	p_tcb->taskProc = selectShield_080 ;
}


static void selectShield_080( SYSTEMTCB *p_tcb )
{
	SYSCHILDTCB		*p_new ;
	LAYOUT			const *p_layout ;
	int				x, y ;


	if( p_tcb->childCount ) {
		return ;
	}

	// 未選択スプライトタスク消去
	task_destroyPrio(TASK_PRIO_SHIELD+(g.powerMeter.shield^2), TASK_PRIO_SHIELD+g.powerMeter.shield) ;

	p_layout = &m_layout[g.powerMeter.playerType/2] ;

	// Shadow
	createPatternTask(p_tcb, patternTask,  TASK_PRIO_MULTIPLE+0,  96, p_layout->multipleY-16, PAT_VIXEN2) ;
	createActionTask(p_tcb,  multipleTask, TASK_PRIO_MULTIPLE+0,  80, p_layout->multipleY,    ACT_MULTIPLE, 0) ;
	createActionTask(p_tcb,  multipleTask, TASK_PRIO_MULTIPLE+0,  56, p_layout->multipleY+16, ACT_MULTIPLE, 0) ;

	// Fixed
	createPatternTask(p_tcb, patternTask,  TASK_PRIO_MULTIPLE+1, 168, p_layout->multipleY,    PAT_VIXEN2) ;
	createActionTask(p_tcb,  multipleTask, TASK_PRIO_MULTIPLE+1, 144, p_layout->multipleY-16, ACT_MULTIPLE, 1) ;
	createActionTask(p_tcb,  multipleTask, TASK_PRIO_MULTIPLE+1, 144, p_layout->multipleY+16, ACT_MULTIPLE, 1) ;

	// Rolling
	createPatternTask(p_tcb, patternTask,  TASK_PRIO_MULTIPLE+2, 240, p_layout->multipleY,    PAT_VIXEN2) ;

	x               = 240                 + getInt( getCos( 224 ) * 24 ) ;
	y               = p_layout->multipleY + getInt( getSin( 224 ) * 24 ) ;
	p_new           = createActionTask(p_tcb,  rollingMultipleTask, TASK_PRIO_MULTIPLE+2, x, y, ACT_MULTIPLE, 2) ;
	p_new->p_layout = p_layout ;
	p_new->angle    = 224 ;

	x               = 240                 + getInt( getCos(  96 ) * 24 ) ;
	y               = p_layout->multipleY + getInt( getSin(  96 ) * 24 ) ;
	p_new           = createActionTask(p_tcb,  rollingMultipleTask, TASK_PRIO_MULTIPLE+2, x, y, ACT_MULTIPLE, 2) ;
	p_new->p_layout = p_layout ;
	p_new->angle    = 96 ;

	//
	createOpenRectTask(p_tcb, &p_layout->multipleRect[0], PALPOS_SELECT,   true) ;
	createOpenRectTask(p_tcb, &p_layout->multipleRect[1], PALPOS_UNSELECT, true) ;
	createOpenRectTask(p_tcb, &p_layout->multipleRect[2], PALPOS_UNSELECT, true) ;

	p_tcb->taskProc = selectMultiple ;
}


// -------------------------------------------------------------------------------- 
//!  マルチプル選択                                                                 
//!                                                                                 
//!  @param [in,out] p_tcb  システムタスク TCB                                      
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void selectMultiple( SYSTEMTCB *p_tcb )
{
	LAYOUT		const *p_layout ;
	uint8_t		selectMultiple ;


	if( p_tcb->childCount ) {
		return ;
	}

	p_layout = &m_layout[g.powerMeter.playerType/2] ;

	// 選択
	if( g.pad.once & (PAD_BTN( 0 )|PAD_BTN( 1 )) ) {
		createBlinkCursorTask(p_tcb, &p_layout->multipleRect[g.powerMeter.multiple]) ;

		p_tcb->taskProc = selectMultiple_040 ;

		sound_requestSE(1, SE_02) ;
		return ;
	}

	// カーソル移動
	selectMultiple = g.powerMeter.multiple ;
	if( (g.powerMeter.multiple!=2) && (g.pad.once&PAD_RIGHT) ) {
		g.powerMeter.multiple++ ;
	} else
	if( (g.powerMeter.multiple!=0) && (g.pad.once&PAD_LEFT) ) {
		g.powerMeter.multiple-- ;
	} else {
		return ;
	}

	moveCursor(&p_layout->multipleRect[selectMultiple], &p_layout->multipleRect[g.powerMeter.multiple]) ;

	sound_requestSE(0, SE_00) ;
}


static void selectMultiple_040( SYSTEMTCB *p_tcb )
{
	RECT	const *p_rect ;
	int		index ;


	if( p_tcb->childCount ) {
		return ;
	}

	p_tcb->taskProc = selectMultiple_080 ;
	p_tcb->timer0   = 10 ;

	p_rect = &m_layout[g.powerMeter.playerType/2].multipleRect[0] ;
	for(index=0; index<3; index++,p_rect++) {
		if( index != g.powerMeter.multiple ) {
			createCloseRectTask(p_tcb, p_rect, PALPOS_UNSELECT, true) ;
		}
	}
}


static void selectMultiple_080( SYSTEMTCB *p_tcb )
{
	LAYOUT	const *p_layout ;


	if( p_tcb->childCount ) {
		return ;
	}

	if( --p_tcb->timer0 ) {
		return ;
	}

	//
	p_layout = &m_layout[g.powerMeter.playerType/2] ;
	createCloseRectTask(p_tcb, &p_layout->multipleRect[g.powerMeter.multiple], PALPOS_RECT, true) ;
	createCloseRectTask(p_tcb, &p_layout->sheildRect[g.powerMeter.shield],     PALPOS_RECT, true) ;
	createCloseRectTask(p_tcb, &m_weaponRect,                                  PALPOS_RECT, true) ;

	p_tcb->taskProc = selectMultiple_0c0 ;
}


static void selectMultiple_0c0( SYSTEMTCB *p_tcb )
{
	if( p_tcb->childCount ) {
		return ;
	}

	setVSyncProc(NULL, NULL) ;

	// 非表示
	vga_setFgBlank() ;
	vga_setSpriteBlank() ;
	vga_setTextBlank() ;

	// タスク破棄
	task_destroyPrio(TASK_PRIO_SYSTEM+1, TASK_PRIO_POST-1) ;

	// VRAM クリア
	setMemory(&g.vram.tile[0][0][0], 0, sizeof( g.vram.tile )) ;

	// フェードアウト
	sound_fadeOutBGM() ;

	p_tcb->taskProc = selectMultiple_100 ;
	p_tcb->timer0    = 60 ;
}


static void selectMultiple_100( SYSTEMTCB *p_tcb )
{
	if( --p_tcb->timer0 ) {
		return ;
	}

	task_setSystemMode( SYSTEM_MODE_GAME ) ;
}
