// -------------------------------------------------------------------------------- 
//!  @file  title.c                                                                 
//!  タイトル画面のソースファイル                                                   
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "title.h"

	//! メニュー項目
	typedef struct {
		uint8_t						x ;								//!< ｘ座標
		uint8_t						y ;								//!< ｙ座標
		char						const *p_string ;				//!< 文字列
		TASKPROC					taskProc ;						//!< タスク処理
	} MENUITEM ;


static void RAMFUNC title_040( SYSTEMTCB *p_tcb ) ;
static void RAMFUNC title_080( SYSTEMTCB *p_tcb ) ;
static void RAMFUNC title_0c0( SYSTEMTCB *p_tcb ) ;
static void RAMFUNC title_100( SYSTEMTCB *p_tcb ) ;

//! メニュー項目
static const MENUITEM				m_menuItem[]={
										{ 15, 23, "GAME START", title_0c0 },
										{ 17, 24, "OPTION",     title_100 },
									} ;


// -------------------------------------------------------------------------------- 
//!  タイトル画面 VBlank 処理                                                       
//!                                                                                 
//!  @param [in] p_tcb  システムタスク TCB                                          
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC title_vblank( SYSTEMTCB *p_tcb )
{
	vga_setFgTile(&g.vram.tile[1][0][0], &g.vram.pat[0][0], &g.vram.pal[0][0]) ;
}


// -------------------------------------------------------------------------------- 
//!  タイトル画面                                                                   
//!                                                                                 
//!  @param [in,out] p_tcb  システムタスク TCB                                      
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
void RAMFUNC title( SYSTEMTCB *p_tcb )
{
	MENUITEM	const *p_menuItem ;
	int			index ;

	// 非表示
	vga_setFgBlank() ;
	vga_setSpriteBlank() ;
	vga_setTextBlank() ;
	vga_setTextOffset(0, 0) ;
	vga_setSpriteOffset(0, 0) ;
	vga_setFgTileOffset(0, 0) ;

	setVSyncProc(NULL, NULL) ;
	vga_setHSyncProc( NULL) ;

	// 初期化
	setMemory(&g.vram.tile[0][0][0], 0, sizeof( g.vram.tile )) ;
	setMemory(&g.vram.pal[0][0],     0, sizeof( g.vram.pal )) ;
	p_tcb->cursor = 0 ;

	// データ転送
	xferData(XFERDATA_FIX_OFFSET, XFERDATA_FIX_COUNT) ;

	// メニュー項目描画
	p_menuItem = &m_menuItem[0] ;
	for(index=0; index<sizeof( m_menuItem )/sizeof( m_menuItem[0] ); index++,p_menuItem++) {
		writeTileString8(p_menuItem->x, p_menuItem->y, &p_menuItem->p_string[0], index==0) ;
	}

	setVSyncProc(title_vblank, p_tcb) ;

	// タスク破棄
	task_destroyPrio(TASK_PRIO_SYSTEM+1, TASK_PRIO_POST-1) ;

	p_tcb->taskProc = title_040 ;
}


static void RAMFUNC title_040( SYSTEMTCB *p_tcb )
{
	MENUITEM	const *p_menuItem ;
	uint8_t		cursor ;


	if( g.pad.once & PAD_RB ) {
		task_setSystemMode( SYSTEM_MODE_GAME ) ;
		return ;
	}

	// カーソル移動
	if( g.pad.once & (PAD_SELECT|PAD_UP|PAD_DOWN) ) {
		cursor = p_tcb->cursor ;
		if( g.pad.once & PAD_UP ) {
			if( --p_tcb->cursor == (uint8_t )-1 ) {
				p_tcb->cursor = sizeof( m_menuItem )/sizeof( m_menuItem[0] ) - 1 ;
			}
		} else {
			if( ++p_tcb->cursor == (sizeof( m_menuItem )/sizeof( m_menuItem[0] )) ) {
				p_tcb->cursor = 0 ;
			}
		}

		if( p_tcb->cursor != cursor ) {
			p_menuItem = &m_menuItem[cursor] ;
			writeTileString8(p_menuItem->x, p_menuItem->y, &p_menuItem->p_string[0], 0) ;

			p_menuItem = &m_menuItem[p_tcb->cursor] ;
			writeTileString8(p_menuItem->x, p_menuItem->y, &p_menuItem->p_string[0], 1) ;

			sound_requestSE(0, SE_00) ;
		}
		return ;
	}

	// 決定
	if( g.pad.once & (PAD_START|PAD_BTN( 0 )|PAD_BTN( 1 )|PAD_BTN( 2 )) ) {
		sound_requestSE(0, SE_02) ;

		p_tcb->taskProc = title_080 ;
		p_tcb->timer0   = 90 ;
	}
}


static void RAMFUNC title_080( SYSTEMTCB *p_tcb )
{
	MENUITEM	const *p_menuItem ;


	p_menuItem = &m_menuItem[p_tcb->cursor] ;

	if( !--p_tcb->timer0 ) {
		p_menuItem->taskProc( p_tcb ) ;
		return ;
	}

	if( g.count1 & 7 ) {
		return ;
	}

	if( p_tcb->timer0 & 8 ) {
		writeTileString8(p_menuItem->x, p_menuItem->y, &p_menuItem->p_string[0], 1) ;
	} else {
		writeTileString8(p_menuItem->x, p_menuItem->y, "          ",             1) ;
	}
}


static void RAMFUNC title_0c0( SYSTEMTCB *p_tcb )
{
	task_setSystemMode( SYSTEM_MODE_POWERMETERSELECT ) ;
}


static void RAMFUNC title_100( SYSTEMTCB *p_tcb )
{
	task_setSystemMode( SYSTEM_MODE_OPTION ) ;
}
