// -------------------------------------------------------------------------------- 
//!  @file  forcefield.c                                                            
//!  フォースフィールドタスク処理のソースファイル                                   
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "vixen.h"
#include "forcefield.h"


// -------------------------------------------------------------------------------- 
//!  フォースフィールド破壊                                                         
//!                                                                                 
//!  @param [in] p_tcb  ヴィクセン武器タスク TCB                                    
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC forcefield_destroy( VIXENWEAPONTCB *p_tcb )
{
	vixen_weapon_destroy( p_tcb ) ;

	g.powerMeter.status |= POWERMETER_STATUS_SHIELD ;
}


// -------------------------------------------------------------------------------- 
//!  フォースフィールド当たり処理                                                   
//!                                                                                 
//!  @param [in,out] p_self   自分                                                  
//!  @param [in,out] p_rival  相手                                                  
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static void RAMFUNC forcefield_hit(HIT *p_self, HIT *p_rival)
{
	VIXENWEAPONTCB	*p_tcb=MEMBERSTRUCT(VIXENWEAPONTCB, obj.hit, p_self) ;


	sound_requestSE(2, SE_30) ;

	if( --p_tcb->p_parent->shieldRemain == 1 ) {
		p_self->flag = 0 ;
		obj_setAction(&p_tcb->obj, ACT_FORCEFIELD0) ;
	}
}


// -------------------------------------------------------------------------------- 
//!  フォースフィールドタスク                                                       
//!                                                                                 
//!  @param [in] p_tcb  ヴィクセン武器タスク TCB                                    
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC forcefield( VIXENWEAPONTCB *p_tcb )
{
	VIXENTCB	*p_parent ;
	uint8_t		attr ;

	// 破壊
	p_parent = p_tcb->p_parent ;
	if( p_parent->shieldRemain <= 0 ) {
		forcefield_destroy( p_tcb ) ;
		return ;
	}

	p_tcb->obj.x = p_parent->obj.x ;
	p_tcb->obj.y = p_parent->obj.y ;

	// 壁
	if( p_tcb->count0 ) {
		p_tcb->count0-- ;
	} else {
		attr  = bg_checkHit(p_tcb->obj.x-toFp( 16 ), p_tcb->obj.y-toFp( 8 )) ;
		attr |= bg_checkHit(p_tcb->obj.x+toFp( 15 ), p_tcb->obj.y-toFp( 8 )) ;
		attr |= bg_checkHit(p_tcb->obj.x-toFp( 16 ), p_tcb->obj.y+toFp( 7 )) ;
		attr |= bg_checkHit(p_tcb->obj.x+toFp( 15 ), p_tcb->obj.y+toFp( 7 )) ;

		if( attr & BG_ATTR_WALL ) {
			sound_requestSE(2, SE_30) ;

			if( !--p_parent->shieldRemain ) {
				forcefield_destroy( p_tcb ) ;
				return ;
			}

			p_tcb->count0 = 10 ;

			if( p_parent->shieldRemain == 1 ) {
				obj_setAction(&p_tcb->obj, ACT_FORCEFIELD0) ;
				return ;
			}
		}
	}

	obj_updateAction( &p_tcb->obj ) ;
}


// -------------------------------------------------------------------------------- 
//!  フォースフィールドタスクを生成                                                 
//!                                                                                 
//!  @param [in] p_parent  親タスク TCB                                             
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
void RAMFUNC forcefield_create( VIXENTCB *p_parent )
{
	VIXENWEAPONTCB	*p_tcb ;

	// タスク生成
	p_tcb                   = task_create((TCB *)p_parent, forcefield, TASK_PRIO_SHIELD) ;
	p_tcb->obj.prio         = OBJ_PRIO2 | 0 ;
	p_tcb->obj.status       = 0 ;
	p_tcb->obj.x            = p_parent->obj.x ;
	p_tcb->obj.y            = p_parent->obj.y ;
	p_tcb->obj.hit.proc     = forcefield_hit ;
	p_tcb->obj.hit.strength = 8 ;
	p_tcb->weapon           = VIXEN_WEAPON_FORCEFIELD ;
	p_tcb->count0           = 0 ;

	obj_setAction(&p_tcb->obj, ACT_FORCEFIELD1) ;

	p_parent->weapon      |= VIXEN_WEAPON_FORCEFIELD ;
	p_parent->shieldRemain = 3 ;
}
