// -------------------------------------------------------------------------------- 
//!  @file  laser.c                                                                 
//!  レーザータスク処理のソースファイル                                             
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "vixen.h"
#include "laser.h"


static void RAMFUNC laser_040( VIXENWEAPONTCB *p_tcb ) ;


// -------------------------------------------------------------------------------- 
//!  レーザー破壊                                                                   
//!                                                                                 
//!  @param [in] p_tcb  ヴィクセン武器タスク TCB                                    
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC laser_destroy( VIXENWEAPONTCB *p_tcb )
{
	p_tcb->taskProc = laser_040 ;
	p_tcb->timer0   = 14 ;
}


// -------------------------------------------------------------------------------- 
//!  レーザー当たり処理                                                             
//!                                                                                 
//!  @param [in,out] p_self   自分                                                  
//!  @param [in,out] p_rival  相手                                                  
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static void RAMFUNC laser_hit(HIT *p_self, HIT *p_rival)
{
	// 破壊不可能でないなら無視
	if( !(p_rival->flag&HIT_FLAG_INDESTRUCTIBLE) ) {
		return ;
	}

	p_self->flag = 0 ;

	laser_destroy( MEMBERSTRUCT(VIXENWEAPONTCB, obj.hit, p_self) ) ;
}


// -------------------------------------------------------------------------------- 
//!  レーザータスク                                                                 
//!                                                                                 
//!  @param [in] p_tcb  ヴィクセン武器タスク TCB                                    
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC laser( VIXENWEAPONTCB *p_tcb )
{
	int24f8_t	x ;

	// ｘ方向移動
	x             = p_tcb->obj.x ;
	p_tcb->obj.x += toFp( 16 ) ;
	if( p_tcb->obj.x >= toFp( SCREEN_WIDTH+8 ) ) {
		laser_destroy( p_tcb ) ;
		return ;
	}

	// ｙ方向移動
	p_tcb->obj.y = p_tcb->p_parent->obj.y ;

	// 壁
	if( bg_checkHitX(x, p_tcb->obj.y, p_tcb->obj.x, &p_tcb->obj.x) & BG_ATTR_WALL ) {
		laser_destroy( p_tcb ) ;

		p_tcb->obj.prio = OBJ_PRIO2 | 0 ;
		obj_setPattern(&p_tcb->obj, PAT_LASER_EFFECT0+(g.count2&3)) ;
		return ;
	}

	obj_setPattern(&p_tcb->obj, p_tcb->obj.pattern) ;
}


static void RAMFUNC laser_040( VIXENWEAPONTCB *p_tcb )
{
	VIXENTCB	*p_parent ;


	if( --p_tcb->timer0 ) {
		return ;
	}

	p_parent = p_tcb->p_parent ;

	if( ++p_parent->laser.destroy == p_parent->laser.count ) {
		p_parent->weapon       &= ~(VIXEN_WEAPON_LASER|VIXEN_WEAPON_SHOOT1|VIXEN_WEAPON_SHOOT0) ;
		p_parent->laser.count   = 0 ;
		p_parent->laser.destroy = 0 ;
	}

	task_destroy( (TCB *)p_tcb ) ;
}


// -------------------------------------------------------------------------------- 
//!  レーザータスクを生成                                                           
//!                                                                                 
//!  @param [in] p_parent  親タスク TCB                                             
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
void RAMFUNC laser_create( VIXENTCB *p_parent )
{
	VIXENWEAPONTCB	*p_tcb ;
	int24f8_t		x ;


	if( !(g.pad.conv&PAD_BTN( 1 )) ) {
		p_parent->weapon &= ~VIXEN_WEAPON_LASER ;
		return ;
	}

	// ｘ座標オフセット
	if( p_parent->prio == TASK_PRIO_PLAYER ) {
		x = p_parent->obj.x + toFp( 8 ) ;
	} else {
		x = p_parent->obj.x + toFp( 12 ) ;
	}

	if( !(p_parent->weapon&(VIXEN_WEAPON_LASER|VIXEN_WEAPON_SHOOT0|VIXEN_WEAPON_SHOOT1)) ) {

		p_parent->weapon       |= (VIXEN_WEAPON_LASER|VIXEN_WEAPON_SHOOT0|VIXEN_WEAPON_SHOOT1) ;
		p_parent->laser.count   = 1 ;
		p_parent->laser.destroy = 0 ;
		p_parent->laser.x       = x ;

		// ヴィクセンのみ効果音
		if( p_parent->prio == TASK_PRIO_PLAYER ) {
			sound_requestSE(1, SE_06) ;
		}

	} else {

		if( !(p_parent->weapon&VIXEN_WEAPON_LASER) ) {
			return ;
		}

		if( ++p_parent->laser.count == 16 ) {
			p_parent->weapon &= ~VIXEN_WEAPON_LASER ;
		}

		if( p_parent->laser.x < x ) {
			p_parent->laser.x = x ;
		} else {
			x = p_parent->laser.x ;
		}

	}

	// タスク生成
	p_tcb                   = task_create((TCB *)p_parent, laser, TASK_PRIO_WEAPON) ;
	p_tcb->obj.prio         = OBJ_PRIO3 | (g.count2 & 7) ;
	p_tcb->obj.status       = 0 ;
	p_tcb->obj.pattern      = PAT_LASER0 + (p_parent->laser.count & 1) ;
	p_tcb->obj.x            = x ;
	p_tcb->obj.y            = p_parent->obj.y ;
	p_tcb->obj.hit.proc     = laser_hit ;
	p_tcb->obj.hit.strength = 2 ;
	p_tcb->weapon           = VIXEN_WEAPON_LASER ;

	obj_setPattern(&p_tcb->obj, p_tcb->obj.pattern) ;
}
