// -------------------------------------------------------------------------------- 
//!  @file  missile.c                                                               
//!  ミサイルタスク処理のソースファイル                                             
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "vixen.h"
#include "missile.h"


static void RAMFUNC missile_040( VIXENWEAPONTCB *p_tcb ) ;
static void RAMFUNC missile_landing_040( VIXENWEAPONTCB *p_tcb ) ;


// -------------------------------------------------------------------------------- 
//!  ミサイル当たり処理                                                             
//!                                                                                 
//!  @param [in,out] p_self   自分                                                  
//!  @param [in,out] p_rival  相手                                                  
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static void RAMFUNC missile_hit(HIT *p_self, HIT *p_rival)
{
	VIXENWEAPONTCB	*p_tcb=MEMBERSTRUCT(VIXENWEAPONTCB, obj.hit, p_self) ;


	p_self->flag = 0 ;

	vixen_weapon_effect(p_tcb, ACT_EFFECT00) ;
}


// -------------------------------------------------------------------------------- 
//!  ミサイルタスク                                                                 
//!                                                                                 
//!  @param [in] p_tcb  ヴィクセン武器タスク TCB                                    
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC missile( VIXENWEAPONTCB *p_tcb )
{
	// 壁
	if( bg_checkHit(p_tcb->obj.x, p_tcb->obj.y) & BG_ATTR_WALL ) {
		vixen_weapon_effect(p_tcb, ACT_EFFECT00) ;
		return ;
	}

	p_tcb->taskProc = missile_040 ;

	missile_040( p_tcb ) ;
}


static void RAMFUNC missile_040( VIXENWEAPONTCB *p_tcb )
{
	int24f8_t	y ;

	// ｙ方向移動
	y             = p_tcb->obj.y ;
	p_tcb->obj.y += getInt( getSin( p_tcb->angle ) * p_tcb->speed ) ;
	if( p_tcb->obj.y >= toFp( SCREEN_HEIGHT+8 ) ) {
		vixen_weapon_destroy( p_tcb ) ;
		return ;
	}

	// 着地
	if( bg_checkHitY(p_tcb->obj.x, y, p_tcb->obj.y, &p_tcb->obj.y) & BG_ATTR_WALL ) {
		p_tcb->obj.status |= OBJ_STATUS_SPRPRIO ;
		p_tcb->taskProc    = missile_landing ;
	}

	// ｘ方向移動
	p_tcb->obj.x += getInt( getCos( p_tcb->angle ) * p_tcb->speed ) ;
	if( p_tcb->obj.x >= toFp( SCREEN_WIDTH+16 ) ) {
		vixen_weapon_destroy( p_tcb ) ;
		return ;
	}

	obj_setPattern(&p_tcb->obj, p_tcb->pattern+(p_tcb->angle/16)) ;
}


// -------------------------------------------------------------------------------- 
//!  ミサイル着地                                                                   
//!                                                                                 
//!  @param [in] p_tcb  ヴィクセン武器タスク TCB                                    
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
void RAMFUNC missile_landing( VIXENWEAPONTCB *p_tcb )
{
	int24f8_t	y ;


	if( p_tcb->angle ) {
		p_tcb->angle -= 8 ;
	}

	// ｙ方向移動
	y             = p_tcb->obj.y ;
	p_tcb->obj.y += getInt( getSin( p_tcb->angle ) * p_tcb->speed ) ;
	if( p_tcb->obj.y >= toFp( SCREEN_HEIGHT+8 ) ) {
		vixen_weapon_destroy( p_tcb ) ;
		return ;
	}

	// 地面なし
	if( !(bg_checkHitY(p_tcb->obj.x, y, p_tcb->obj.y, &p_tcb->obj.y)&BG_ATTR_WALL) ) {
		p_tcb->obj.status |= OBJ_STATUS_SPRPRIO ;
		p_tcb->taskProc    = missile_landing_040 ;
	}

	// ｘ方向移動
	p_tcb->obj.x += getInt( getCos( p_tcb->angle ) * p_tcb->speed ) ;
	if( p_tcb->obj.x >= toFp( SCREEN_WIDTH+16 ) ) {
		vixen_weapon_destroy( p_tcb ) ;
		return ;
	}

	// 壁
	if( bg_checkHit(p_tcb->obj.x, p_tcb->obj.y) & BG_ATTR_WALL ) {
		vixen_weapon_effect(p_tcb, ACT_EFFECT00) ;
		return ;
	}

	obj_setPattern(&p_tcb->obj, p_tcb->pattern+(p_tcb->angle/16)) ;
}


static void RAMFUNC missile_landing_040( VIXENWEAPONTCB *p_tcb )
{
	int24f8_t	y ;


	if( p_tcb->angle != 64 ) {
		p_tcb->angle += 8 ;
	}

	// ｙ方向移動
	y             = p_tcb->obj.y ;
	p_tcb->obj.y += getInt( getSin( p_tcb->angle ) * p_tcb->speed ) ;
	if( p_tcb->obj.y >= toFp( SCREEN_HEIGHT+8 ) ) {
		vixen_weapon_destroy( p_tcb ) ;
		return ;
	}

	// 着地
	if( bg_checkHitY(p_tcb->obj.x, y, p_tcb->obj.y, &p_tcb->obj.y) & BG_ATTR_WALL ) {
		p_tcb->obj.status &= ~OBJ_STATUS_SPRPRIO ;
		p_tcb->taskProc    = missile_landing ;
	}

	// ｘ方向移動
	p_tcb->obj.x += getInt( getCos( p_tcb->angle ) * p_tcb->speed ) ;
	if( p_tcb->obj.x >= toFp( SCREEN_WIDTH+16 ) ) {
		vixen_weapon_destroy( p_tcb ) ;
		return ;
	}

	// 壁
	if( bg_checkHit(p_tcb->obj.x, p_tcb->obj.y) & BG_ATTR_WALL ) {
		vixen_weapon_effect(p_tcb, ACT_EFFECT00) ;
		return ;
	}

	obj_setPattern(&p_tcb->obj, p_tcb->pattern+(p_tcb->angle/16)) ;
}


// -------------------------------------------------------------------------------- 
//!  ミサイルタスクを生成                                                           
//!                                                                                 
//!  @param [in] p_parent  親タスク TCB                                             
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
void RAMFUNC missile_create( VIXENTCB *p_parent )
{
	VIXENWEAPONTCB	*p_tcb ;


	if( !(g.pad.conv&PAD_BTN( 2 )) ) {
		return ;
	}

	if( p_parent->weapon & VIXEN_WEAPON_MISSILE ) {
		return ;
	}

	// タスク生成
	p_tcb                   = task_create((TCB *)p_parent, missile, TASK_PRIO_WEAPON) ;
	p_tcb->obj.prio         = OBJ_PRIO3 | 4 ;
	p_tcb->obj.status       = 0 ;
	p_tcb->obj.x            = p_parent->obj.x ;
	p_tcb->obj.y            = p_parent->obj.y ;
	p_tcb->obj.hit.proc     = missile_hit ;
	p_tcb->obj.hit.strength = 32 ;
	p_tcb->weapon           = VIXEN_WEAPON_MISSILE ;
	p_tcb->speed            = toFp( 3 ) ;
	p_tcb->angle            = 48 ;
	p_tcb->pattern          = PAT_MISSILE0 ;

	obj_setPattern(&p_tcb->obj, p_tcb->pattern+(p_tcb->angle / 16)) ;

	p_parent->weapon |= VIXEN_WEAPON_MISSILE ;

	// ヴィクセンのみ効果音
	if( p_parent->prio == TASK_PRIO_PLAYER ) {
		sound_requestSE(1, SE_04) ;
	}
}
