// -------------------------------------------------------------------------------- 
//!  @file  multiple_rolling.c                                                      
//!  マルチプル・ローリングタスク処理のソースファイル                               
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "vixen.h"
#include "multiple_rolling.h"


// -------------------------------------------------------------------------------- 
//!  マルチプル・ローリングタスク                                                   
//!                                                                                 
//!  @param [in] p_tcb  ヴィクセンマルチプルタスク TCB                              
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC multiple_rolling( VIXENMULTIPLETCB *p_tcb )
{
	VIXENTCB	*p_parent ;


	p_parent = p_tcb->p_parent ;

	// 移動
	p_tcb->posIndex += 5 ;
	p_tcb->obj.x     = p_parent->obj.x + (getCos( p_tcb->posIndex ) * 32) ;
	p_tcb->obj.y     = p_parent->obj.y + (getSin( p_tcb->posIndex ) * 32) ;

	// ショット
	p_parent->shotProc( p_tcb ) ;

	// ミサイル
	p_parent->missileProc( p_tcb ) ;

	obj_updateAction( &p_tcb->obj ) ;
}


// -------------------------------------------------------------------------------- 
//!  マルチプル・ローリングタスクを生成                                             
//!                                                                                 
//!  @param [in] p_parent  親タスク TCB                                             
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
void RAMFUNC multiple_rolling_create( VIXENTCB *p_parent )
{
	VIXENMULTIPLETCB	*p_tcb ;
	uint8_t				posIndex ;


	posIndex = (uint8_t )((g.vixen.multipleCount++ * 128) + (g.count1 * 5)) ;

	p_tcb             = task_create((TCB *)p_parent, multiple_rolling, TASK_PRIO_MULTIPLE) ;
	p_tcb->obj.prio   = OBJ_PRIO2 | 4 ;
	p_tcb->obj.status = 0 ;
	p_tcb->obj.x      = p_parent->obj.x + (getCos( posIndex ) * 32) ;
	p_tcb->obj.y      = p_parent->obj.y + (getSin( posIndex ) * 32) ;
	p_tcb->weapon     = 0 ;
	p_tcb->posIndex   = posIndex ;

	obj_setAction(&p_tcb->obj, ACT_MULTIPLE) ;
}
