// -------------------------------------------------------------------------------- 
//!  @file  twowaymissile.c                                                         
//!  ツーウェイ・ミサイルタスク処理のソースファイル                                 
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "vixen.h"
#include "twowaymissile.h"


static void RAMFUNC twowaymissile_040( VIXENWEAPONTCB *p_tcb ) ;


// -------------------------------------------------------------------------------- 
//!  ツーウェイ・ミサイル当たり処理                                                 
//!                                                                                 
//!  @param [in,out] p_self   自分                                                  
//!  @param [in,out] p_rival  相手                                                  
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
static void RAMFUNC twowaymissile_hit(HIT *p_self, HIT *p_rival)
{
	VIXENWEAPONTCB	*p_tcb=MEMBERSTRUCT(VIXENWEAPONTCB, obj.hit, p_self) ;


	p_self->flag = 0 ;

	vixen_weapon_effect(p_tcb, ACT_EFFECT00) ;
}


// -------------------------------------------------------------------------------- 
//!  ツーウェイ・ミサイルタスク処理                                                 
//!                                                                                 
//!  @param [in] p_tcb  ヴィクセン武器タスク TCB                                    
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
static void RAMFUNC twowaymissile( VIXENWEAPONTCB *p_tcb )
{
	// 壁
	if( bg_checkHit(p_tcb->obj.x, p_tcb->obj.y) & BG_ATTR_WALL ) {
		vixen_weapon_effect(p_tcb, ACT_EFFECT00) ;
		return ;
	}

	p_tcb->taskProc = twowaymissile_040 ;

	twowaymissile_040( p_tcb ) ;
}


static void RAMFUNC twowaymissile_040( VIXENWEAPONTCB *p_tcb )
{
	// ｘ方向移動
	p_tcb->obj.x += getInt( getCos( p_tcb->angle ) * p_tcb->speed ) ;
	if( p_tcb->obj.x >= toFp( SCREEN_WIDTH+8 ) ) {
		vixen_weapon_destroy( p_tcb ) ;
		return ;
	}

	// y方向移動
	p_tcb->obj.y += getInt( getSin( p_tcb->angle ) * p_tcb->speed ) ;
	if( (uint32_t )(getInt( p_tcb->obj.y )+8) >= (SCREEN_HEIGHT+(8*2)) ) {
		vixen_weapon_destroy( p_tcb ) ;
		return ;
	}

	// 壁
	if( bg_checkHit(p_tcb->obj.x, p_tcb->obj.y) & BG_ATTR_WALL ) {
		vixen_weapon_effect(p_tcb, ACT_EFFECT00) ;
		return ;
	}

	obj_setPattern(&p_tcb->obj, p_tcb->pattern) ;
}


// -------------------------------------------------------------------------------- 
//!  ツーウェイ・ミサイルタスクを生成                                               
//!                                                                                 
//!  @param [in] p_parent  親タスク TCB                                             
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
void RAMFUNC twowaymissile_create( VIXENTCB *p_parent )
{
	VIXENWEAPONTCB	*p_tcb ;


	if( !(g.pad.conv&PAD_BTN( 2 )) ) {
		return ;
	}

	if( p_parent->weapon & VIXEN_WEAPON_MISSILE ) {
		return ;
	}

	// タスク生成
	p_tcb                   = task_create((TCB *)p_parent, twowaymissile, TASK_PRIO_WEAPON) ;
	p_tcb->obj.prio         = OBJ_PRIO3 | 4 ;
	p_tcb->obj.status       = 0 ;
	p_tcb->obj.x            = p_parent->obj.x ;
	p_tcb->obj.y            = p_parent->obj.y ;
	p_tcb->obj.hit.proc     = twowaymissile_hit ;
	p_tcb->obj.hit.strength = 40 ;
	p_tcb->weapon           = VIXEN_WEAPON_MISSILE ;
	p_tcb->speed            = toFp( 2.5 ) ;
	p_tcb->angle            = 8 ;

	if( getInt( p_tcb->obj.y ) < (SCREEN_HEIGHT/2) ) {	// 上方向
		p_tcb->angle   = -40 ;
		p_tcb->pattern = PAT_TWOWAYMISSILEU3 ;
	} else {											// 下方向
		p_tcb->angle   =  40 ;
		p_tcb->pattern = PAT_TWOWAYMISSILED3 ;
	}

	obj_setPattern(&p_tcb->obj, p_tcb->pattern) ;

	p_parent->weapon |= VIXEN_WEAPON_MISSILE ;

	// ヴィクセンのみ効果音
	if( p_parent->prio == TASK_PRIO_PLAYER ) {
		sound_requestSE(1, SE_04) ;
	}
}
