// -------------------------------------------------------------------------------- 
//!  @file  vixen.h                                                                 
//!  ヴィクセンタスク処理のヘッダファイル                                           
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#ifndef __VIXEN__VIXEN_H__INCLUDED__
#define __VIXEN__VIXEN_H__INCLUDED__


#include "bg/bg.h"


typedef struct _vixentcb			VIXENTCB ;						//!< ヴィクセンタスク TCB
typedef struct _vixentcb			VIXENMULTIPLETCB ;				//!< ヴィクセンマルチプルタスク TCB

	//! ヴィクセン武器生成ビット
	enum {
		VIXEN_WEAPON_SHOOT0			=	1 << 0,						//!< ショット０
		VIXEN_WEAPON_SHOOT1			=	1 << 1,						//!< ショット１
		VIXEN_WEAPON_LASER			=	1 << 2,						//!< レーザー
		VIXEN_WEAPON_MISSILE		=	1 << 3,						//!< ミサイル
		VIXEN_WEAPON_SHIELD0		=	1 << 4,						//!< シールド０
		VIXEN_WEAPON_SHIELD1		=	1 << 5,						//!< シールド１
		VIXEN_WEAPON_FORCEFIELD		=	VIXEN_WEAPON_SHIELD0 | VIXEN_WEAPON_SHIELD1,
																	//!< フォースフィールド
	} ;

	//! ヴィクセン武器タスク TCB
	typedef struct {
		TCBHEAD( VIXENTCB )											//!< TCB ヘッダ
		OBJ							obj ;							//!< オブジェクト
		uint8_t						weapon ;						//!< 武器ビット
		int8f8_t					speed ;							//!< 速度
		uint8_t						angle ;							//!< 角度
		uint8_t						action ;						//!< 基準アクション番号
		uint16_t					pattern ;						//!< 基準パターン番号
		int24f8_t					xofs ;							//!< ｘ座標オフセット
		int24f8_t					yofs ;							//!< ｙ座標オフセット
		uint8_t						flag ;							//!< 汎用フラグ
	} VIXENWEAPONTCB ;

	//! ヴィクセン・ヴィクセンマルチプルタスク TCB
	struct _vixentcb {
		TCBHEAD( VIXENTCB )											//!< TCB ヘッダ
		OBJ							obj ;							//!< オブジェクト
		TASKPROC					shotProc ;						//!< ショットタスク処理
		TASKPROC					missileProc ;					//!< ミサイルタスク処理
		uint8_t						weapon ;						//!< 武器表示ビット
		int8_t						tilt ;							//!< 傾き
		uint8_t						speedCount ;					//!< 速度数
		int8f8_t					speed ;							//!< 速度
		uint8_t						posIndex ;						//!< 位置インデックス
		int8_t						shieldRemain ;					//!< シールド残り耐久度
		uint8_t						shieldAction ;					//!< シールドアクションオフセット

		//! レーザー
		struct {
			uint8_t					count ;							//!< 生成数
			uint8_t					destroy ;						//!< 破棄数
			int24f8_t				x ;								//!< 表示開始ｘ座標
		} laser ;
	} ;


extern void         vixen_create( SYSTEMTCB *p_tcb ) ;
extern void RAMFUNC vixen_weapon_destroy( VIXENWEAPONTCB *p_tcb ) ;
extern void RAMFUNC vixen_weapon_effect(VIXENWEAPONTCB *p_tcb, uint8_t action) ;


#endif//__VIXEN__VIXEN_H__INCLUDED__
