// -------------------------------------------------------------------------------- 
//!  @file  bg.c                                                                    
//!  背景管理のソースファイル                                                       
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "bg.h"
#include "enemy/enemy.h"


static uint16_t						const *m_p_mapData ;				//!< カレントマップデータ
static uint8_t						const *m_p_enemyData ;				//!< カレント敵データ
static uint8_t						m_enemyCount ;						//!< 敵発生カウンタ数


// -------------------------------------------------------------------------------- 
//!  初期化                                                                         
//!                                                                                 
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
void bg_init( void )
{
	// 背景
	g.bg.x.n       = 0 ;
	g.bg.y.n       = 0 ;
	g.bg.xofs      = 0 ;
	g.bg.yofs      = 0 ;
	m_p_mapData    = &g_mapData[0][0] ;

	// 敵
	m_p_enemyData = &g_enemyData[0] ;
	m_enemyCount  = *m_p_enemyData++ ;
}


// -------------------------------------------------------------------------------- 
//!  状態を更新                                                                     
//!                                                                                 
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
void RAMFUNC bg_update( void )
{
	uint8_t		y ;
	int			x ;

	// マップデータ終端
	if( m_p_mapData == &g_mapData[0][sizeof( g_mapData[0] )/sizeof( g_mapData[0][0] )-DRAW_WIDTH] ) {
//#
		static uint8_t	play=0 ;

		if( !play ) {
			play = 1 ;
			sound_playBGM( 1 ) ;
		}
//#

		g.bg.xofs = 0 ;
		return ;
	}

	// スクロール
	x            = g.bg.x.h ;
	g.bg.x.n    += toFp( 0.07 ) ;
	g.bg.xofs    = (g.bg.x.h - x) ;
	m_p_mapData += g.bg.xofs ;

	if( !g.bg.xofs ) {
		return ;
	}

	// 敵
	if( !m_enemyCount ) {
		return ;
	}

	if( --m_enemyCount ) {
		return ;
	}

	y = *m_p_enemyData++ ;

	while( 1 ) {
		enemy_create(y, *m_p_enemyData++) ;

		m_enemyCount = *m_p_enemyData++ ;
		if( !(m_enemyCount&0x80) ) {
			break ;
		}

		y = m_enemyCount & ~0x80 ;
	}
}


// -------------------------------------------------------------------------------- 
//!  テキスト VRAM へ描画                                                           
//!                                                                                 
//!  @param [out] p_txt  テキスト VRAM                                              
//!  @return             なし                                                       
// -------------------------------------------------------------------------------- 
void RAMFUNC bg_draw( uint16_t *p_txt )
{
	uint16_t	const *p_srce, *p_srceEnd ;
	uint16_t	*p_txtEnd ;


	p_srce   = &m_p_mapData[0] ;
	p_txtEnd = &g.vram.txt[DRAW_HEIGHT][0] ;

	do {
		p_srceEnd = &p_srce[DRAW_WIDTH] ;
		do {
			p_txt[0] = p_srce[0] ;
			p_txt[1] = p_srce[1] ;
			p_txt[2] = p_srce[2] ;
			p_txt[3] = p_srce[3] ;
			p_txt[4] = p_srce[4] ;
			p_txt[5] = p_srce[5] ;
			p_txt[6] = p_srce[6] ;
			p_txt[7] = p_srce[7] ;
			p_txt   += 8 ;
			p_srce  += 8 ;
		} while( p_srce != p_srceEnd ) ;
		p_srce += (sizeof( g_mapData[0] ) / sizeof( g_mapData[0][0] ) - DRAW_WIDTH) ;
	} while( p_txt != p_txtEnd ) ;
}
