// -------------------------------------------------------------------------------- 
//!  @file  bullet0.c                                                               
//!  弾 0 タスクのソースファイル                                                    
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "enemy.h"
#include "bullet0.h"


// -------------------------------------------------------------------------------- 
//!  弾 0 タスク処理                                                                
//!                                                                                 
//!  @param [in,out] p_tcb  タスクコントロールブロック                              
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void RAMFUNC bullet0( TCB *p_tcb )
{
	static uint16_t		pattern[]={
							0x20ec,
							0x60ec,
						} ;
	int8_t				x, y ;

	// 移動
	p_tcb->x.n += (getCos( p_tcb->angle ).n * p_tcb->speed / 16) ;
	p_tcb->y.n += (getSin( p_tcb->angle ).n * p_tcb->speed / 16) ;

	// 画面外
	x = p_tcb->x.h - g.bg.x.h ;
	y = p_tcb->y.h - g.bg.y.h ;
	if( ((uint8_t )x>=DRAW_WIDTH) || ((uint8_t )y>=DRAW_HEIGHT) ) {
		enemy_destroy( p_tcb ) ;
		return ;
	}

	// 壁
	if( p_tcb->timer0 ) {		// 一定時間壁をすり抜け
		p_tcb->timer0-- ;
	} else {
		if( g.vram.txt[y][x] & WALL ) {
			enemy_destroy( p_tcb ) ;
			return ;
		}
	}

	g.vram.txt[y][x] = pattern[(p_tcb->count0/4)&1] ;
	p_tcb->count0++ ;
}


// -------------------------------------------------------------------------------- 
//!  弾 0 タスクを生成                                                              
//!                                                                                 
//!  @param [in] sx     開始ｘ座標                                                  
//!  @param [in] sy     開始ｙ座標                                                  
//!  @param [in] tx     目標ｘ座標                                                  
//!  @param [in] ty     目標ｙ座標                                                  
//!  @param [in] speed  速度                                                        
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
void RAMFUNC bullet0_create(uint8_t sx, uint8_t sy, uint8_t tx, uint8_t ty, uint8_t speed)
{
	TCB		*p_tcb ;


	p_tcb            = task_getTcb() ;
	p_tcb->taskProc  = bullet0 ;
	p_tcb->angle     = getAngle(sx, sy, tx, ty) ;
	p_tcb->speed     = speed ;
	p_tcb->count0    = 0 ;
	p_tcb->timer0    = 8 ;
	p_tcb->x.n       = g.bg.x.n + toFp( sx ) ;
	p_tcb->y.n       = g.bg.y.n + toFp( sy ) ;

	task_attachTail(&g.enemyList, p_tcb) ;
}
