// -------------------------------------------------------------------------------- 
//!  @file  enemy00.c                                                               
//!  敵 00 タスクのソースファイル                                                   
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "enemy.h"


// -------------------------------------------------------------------------------- 
//!  敵 00 壁当たりチェック                                                         
//!                                                                                 
//!  @param [in] x      ｘ座標                                                      
//!  @param [in] y      ｙ座標                                                      
//!  @param [in] angle  角度                                                        
//!  @retval     true   壁あり                                                      
//!  @retval     false  壁なし                                                      
// -------------------------------------------------------------------------------- 
static bool RAMFUNC enemy00_hitCheck(int8_t x, int8_t y, uint8_t angle)
{
	int8_t		width ;
	uint16_t	txt, *p_txt, *p_txtEnd ;


	if( !(angle&64) ) {				// 左右

		y -= 3 ;
		if( y < 0 ) {
			return false ;
		}

		if( !angle ) {				// 右へ
			x += 4 ;
			if( x >= DRAW_WIDTH ) {
				return false ;
			}
		} else {					// 左へ
			x -= 4 ;
			if( x < 0 ) {
				return false ;
			}
		}

		p_txt = &g.vram.txt[y][x] ;
		txt   = p_txt[SCREEN_WIDTH*0] |
		        p_txt[SCREEN_WIDTH*1] |
		        p_txt[SCREEN_WIDTH*2] |
		        p_txt[SCREEN_WIDTH*3] |
		        p_txt[SCREEN_WIDTH*4] |
		        p_txt[SCREEN_WIDTH*5] ;

	} else {						// 上下

		x -= 3 ;
		if( x < 0 ) {

			// 左画面外
			width = 6 + x ;
			if( width <= 0 ) {
				return false ;
			}
			x = 0 ;

		} else {

			// 右画面外
			width = DRAW_WIDTH - x ;
			if( width <= 0 ) {
				return false ;
			}

			if( width > 6 ) {
				width = 6 ;
			}

		}

		if( !(angle-64) ) {			// 下へ
			y += 3 ;
			if( y >= DRAW_HEIGHT ) {
				return false ;
			}
		} else {					// 上へ
			y -= 4 ;
			if( y < 0 ) {
				return false ;
			}
		}

		p_txt    = &g.vram.txt[y][x] ;
		p_txtEnd = &p_txt[width] ;

		txt = *p_txt++ ;

		while( p_txt != p_txtEnd ) {
			txt |= *(++p_txt) ;
		}
	}

	if( txt & WALL ) {
		return true ;
	}

	return false ;
}


// -------------------------------------------------------------------------------- 
//!  敵 00 タスク処理                                                               
//!                                                                                 
//!  @param [in,out] p_tcb  タスクコントロールブロック                              
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void RAMFUNC enemy00_040( TCB *p_tcb ) ;

static void RAMFUNC enemy00( TCB *p_tcb )
{
	if( !p_tcb->timer0 ) {
		p_tcb->taskProc = enemy00_040 ;
		enemy00_040( p_tcb ) ;
		return ;
	}

	if( g.bg.xofs ) {
		p_tcb->timer0-- ;
	}
}


static void RAMFUNC enemy00_040( TCB *p_tcb )
{
	static int16_t	add[][2]={
						{ toFp(  0.22 ), toFp(  0 )    },	// 右
						{ toFp(  0 ),    toFp(  0.22 ) },	// 下
						{ toFp( -0.22 ), toFp(  0 )    },	// 左
						{ toFp(  0 ),    toFp( -0.22 ) }	// 上
					} ;
	int8_t			x, y ;

	// 移動
	p_tcb->x.n += add[p_tcb->angle/64][0] ;
	p_tcb->y.n += add[p_tcb->angle/64][1] ;

	x = p_tcb->x.h - g.bg.x.h ;
	y = p_tcb->y.h - g.bg.y.h ;

	obj_updateAction( p_tcb ) ;

	if( obj_drawPattern(x, y, p_tcb->pattern) == false ) {
		enemy_destroy( p_tcb ) ;
		return ;
	}

	// 画面外
	if( ((uint8_t )x >= DRAW_WIDTH) || ((uint8_t )y >= DRAW_HEIGHT) ) {
		return ;
	}

	// 壁
	if( enemy00_hitCheck(x, y, p_tcb->angle) != false ) {
		do {
			// 以前と同じ回転
			p_tcb->angle += p_tcb->count0 ;
			if( enemy00_hitCheck(x, y, p_tcb->angle) == false ) {
				break ;
			}

			// 以前と逆回転
			p_tcb->angle ^= 128 ;
			if( enemy00_hitCheck(x, y, p_tcb->angle) == false ) {
				p_tcb->count0 = -p_tcb->count0 ;
				break ;
			}

			// 反転
			p_tcb->angle = -p_tcb->angle ;
		} while( 0 ) ;
		return ;
	}
}


// -------------------------------------------------------------------------------- 
//!  敵 00 タスクを生成                                                             
//!                                                                                 
//!  @param [in] y     ｙ座標                                                       
//!  @param [in] data  データ                                                       
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
void RAMFUNC enemy00_create(uint8_t y, uint8_t data)
{
	TCB		*p_tcb ;


	p_tcb            = task_getTcb() ;
	p_tcb->taskProc  = enemy00 ;
	p_tcb->angle     = ((data >> 4) & 3) * 64 ;
	p_tcb->count0    = -64 ;

	switch( p_tcb->angle / 64 ) {
		case 0:		// 右
		{
			p_tcb->x.n    = g.bg.x.n + toFp( -1 ) ;
			p_tcb->y.n    = g.bg.y.n + toFp( y ) ;
			p_tcb->timer0 = ((data >> 6) & 3) * 8 ;
			break ;
		}

		case 1:		// 下
		{
			p_tcb->x.n    = g.bg.x.n + toFp( DRAW_WIDTH ) ;
			p_tcb->y.n    = g.bg.y.n + toFp( -1 ) ;
			p_tcb->timer0 = 6 + (((data >> 6) & 3) * 8) ;
			break ;
		}

		case 2:		// 左
		{
			p_tcb->x.n    = g.bg.x.n + toFp( DRAW_WIDTH ) ;
			p_tcb->y.n    = g.bg.y.n + toFp( y ) ;
			p_tcb->timer0 = ((data >> 6) & 3) * 8 ;
			break ;
		}

//		case 3:
		default:	// 上
		{
			p_tcb->x.n    = g.bg.x.n + toFp( DRAW_WIDTH ) ;
			p_tcb->y.n    = g.bg.y.n + toFp( DRAW_HEIGHT-2 ) ;
			p_tcb->timer0 = 6 + (((data >> 6) & 3) * 8) ;
			break ;
		}
	}

	obj_setAction(p_tcb, ACT_ENEMY0) ;

	task_attachHead(&g.enemyList, p_tcb) ;
}
