// -------------------------------------------------------------------------------- 
//!  @file  enemy01.c                                                               
//!  敵 01 タスクのソースファイル                                                   
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "enemy.h"
#include "bullet0.h"


// -------------------------------------------------------------------------------- 
//!  敵 01 タスク処理                                                               
//!                                                                                 
//!  @param [in,out] p_tcb  タスクコントロールブロック                              
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void RAMFUNC enemy01_x( TCB *p_tcb )
{
	FI8		ox ;
	int8_t	x, y, xofs ;

	// 移動
	ox = p_tcb->x ;
	if( !(p_tcb->angle&128) ) {
		p_tcb->x.n += toFp( 0.1 ) ;
		xofs        =  1 ;
	} else {
		p_tcb->x.n -= toFp( 0.1 ) ;
		xofs        = -1 ;
	}

	x = p_tcb->x.h - g.bg.x.h ;
	y = p_tcb->y.h - g.bg.y.h ;

	if( x < 1 ) {
		enemy_destroy( p_tcb ) ;
		return ;
	}

	if( !(g.vram.txt[y][x+xofs]&WALL) ) {
		p_tcb->angle ^= 128 ;
		p_tcb->x      = ox ;
		obj_setAction(p_tcb, ACT_ENEMY1_R+(p_tcb->angle/64)) ;
	} else {
		obj_updateAction( p_tcb ) ;
	}

	if( obj_drawPattern(x, y, p_tcb->pattern) == false ) {
		enemy_destroy( p_tcb ) ;
		return ;
	}

	if( !--p_tcb->timer0 ) {
		p_tcb->timer0 = 200 ;
		bullet0_create(x, y, g.playerTcb.x.h, g.playerTcb.y.h, 4) ;
	}
}


static void RAMFUNC enemy01_y( TCB *p_tcb )
{
	FI8		oy ;
	int8_t	x, y, yofs ;

	// 移動
	oy = p_tcb->y ;
	if( !(p_tcb->angle&128) ) {
		p_tcb->y.n += toFp( 0.1 ) ;
		yofs        =  1 ;
	} else {
		p_tcb->y.n -= toFp( 0.1 ) ;
		yofs        = -1 ;
	}

	x = p_tcb->x.h - g.bg.x.h ;
	y = p_tcb->y.h - g.bg.y.h ;

	if( x < 1 ) {
		enemy_destroy( p_tcb ) ;
		return ;
	}

	if( y < 1 ) {
		p_tcb->y.n   = toFp( 2 ) ;
		p_tcb->angle = 64 ;
		obj_updateAction( p_tcb ) ;
	} else
	if( y > (DRAW_HEIGHT-1) ) {
		p_tcb->y.n   = toFp( DRAW_HEIGHT-1 ) ;
		p_tcb->angle = 192 ;
		obj_updateAction( p_tcb ) ;
	} else
	if( !(g.vram.txt[y+yofs][x]&WALL) ) {
		p_tcb->angle ^= 128 ;
		p_tcb->y      = oy ;
		obj_setAction(p_tcb, ACT_ENEMY1_R+(p_tcb->angle/64)) ;
	} else {
		obj_updateAction( p_tcb ) ;
	}

	if( obj_drawPattern(x, y, p_tcb->pattern) == false ) {
		enemy_destroy( p_tcb ) ;
		return ;
	}

	if( !--p_tcb->timer0 ) {
		p_tcb->timer0 = 200 ;
		bullet0_create(x, y, g.playerTcb.x.h, g.playerTcb.y.h, 4) ;
	}
}


// -------------------------------------------------------------------------------- 
//!  敵 01 タスクを生成                                                             
//!                                                                                 
//!  @param [in] y     ｙ座標                                                       
//!  @param [in] data  データ                                                       
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
void RAMFUNC enemy01_create(uint8_t y, uint8_t data)
{
	TCB		*p_tcb ;


	p_tcb         = task_getTcb() ;
	p_tcb->x.n    = g.bg.x.n + toFp( DRAW_WIDTH ) ;
	p_tcb->y.n    = g.bg.y.n + toFp( y ) ;
	p_tcb->angle  = ((data >> 4) & 3) * 64 ;
	p_tcb->timer0 = 100 + ((p_tcb->angle / 64) * 4) ;

	if( !(p_tcb->angle&64) ) {	// ｘ方向
		p_tcb->taskProc = enemy01_x ;
	} else {					// ｙ方向
		p_tcb->taskProc = enemy01_y ;
	}

	obj_setAction(p_tcb, ACT_ENEMY1_R+(p_tcb->angle/64)) ;

	task_attachHead(&g.enemyList, p_tcb) ;
}
