// -------------------------------------------------------------------------------- 
//!  @file  main.c                                                                  
//!  メイン処理のソースファイル                                                     
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "bg.h"
#include "player/player.h"
#include "player/force.h"
#include "player/bit.h"


// -------------------------------------------------------------------------------- 
//!  文字列を描画                                                                   
//!                                                                                 
//!  @param [in] x         ｘ座標                                                   
//!  @param [in] y         ｙ座標                                                   
//!  @param [in] p_string  文字列                                                   
//!  @param [in] color     色                                                       
//!  @return               なし                                                     
// -------------------------------------------------------------------------------- 
static void drawString(uint8_t x, uint8_t y, char const *p_string, uint8_t color)
{
	uint8_t		code ;
	uint16_t	data, *p_txt ;


	p_txt = &g.vram.txt[y][x] ;
	data  = (uint16_t )color << VGATXT_PC_BIT ;

	while( 1 ) {
		code = *p_string++ ;
		if( code == '\0' ) {
			break ;
		}

		*p_txt++ = data | (uint8_t )code ;
	}
}


// -------------------------------------------------------------------------------- 
//!  ステータスを描画                                                               
//!                                                                                 
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
static void drawStatus( void )
{
	uint16_t	*p_txt, *p_txtEnd ;
	int			count ;

	// 残機
	p_txt    = &g.vram.txt[28][1] ;
	p_txt[0] = 0x70eb ;
	p_txt[1] = 0x70eb ;

	// 得点
	p_txt    = &g.vram.txt[29][3] ;
	p_txt[0] = '1' | (5<<12) ;
	p_txt[1] = 'P' | (5<<12) ;
	p_txt[2] = '-' | (5<<12) ;
	p_txt[3] = '1' | (7<<12) ;
	p_txt[4] = '2' | (7<<12) ;
	p_txt[5] = '3' | (7<<12) ;
	p_txt[6] = '4' | (7<<12) ;
	p_txt[7] = '5' | (7<<12) ;
	p_txt[8] = '0' | (7<<12) ;
	p_txt[9] = '0' | (7<<12) ;

	p_txt    = &g.vram.txt[29][16] ;
	p_txt[0] = 'H' | (5<<12) ;
	p_txt[1] = 'I' | (5<<12) ;
	p_txt[2] = '-' | (5<<12) ;
	p_txt[3] = '5' | (7<<12) ;
	p_txt[4] = '6' | (7<<12) ;
	p_txt[5] = '7' | (7<<12) ;
	p_txt[6] = '8' | (7<<12) ;
	p_txt[7] = '9' | (7<<12) ;
	p_txt[8] = '0' | (7<<12) ;
	p_txt[9] = '0' | (7<<12) ;

	// ビーム
	p_txt     = &g.vram.txt[28][10] ;
	p_txt[0]  = 'B' | (5<<12) ;
	p_txt[1]  = 'E' | (5<<12) ;
	p_txt[2]  = 'A' | (5<<12) ;
	p_txt[3]  = 'M' | (5<<12) ;
	p_txt[4]  = '[' | (7<<12) ;
	p_txt[15] = ']' | (7<<12) ;

	p_txt    = &g.vram.txt[28][15] ;
	p_txtEnd = &g.vram.txt[28][25] ;

	count = g.playerTcb.count0 / 8 ;
	while( count-- ) {
		*p_txt++ = 0x87 | (5<<12) ;
	}

	if( g.playerTcb.count0 & 7 ) {
		*p_txt++ = (0x88 - 1 + (g.playerTcb.count0 & 7)) | (5<<12) ;
	}

	while( p_txt != p_txtEnd ) {
		*p_txt++ = 0x00 ;
	}

#ifdef  DEBUG
		g.vram.txt[0][0]   = 'H' | (4<<12) ;
		g.vram.txt[0][39]  = 'H' | (4<<12) ;
		g.vram.txt[29][0]  = 'H' | (4<<12) ;
		g.vram.txt[29][39] = 'H' | (4<<12) ;
#endif//DEBUG
}


// -------------------------------------------------------------------------------- 
//!  メイン処理                                                                     
//!                                                                                 
//!  @param [in] argc  引数の数                                                     
//!  @param [in] argv  引数                                                         
//!  @return           常に０                                                       
// -------------------------------------------------------------------------------- 
int RAMFUNC main(int argc, char *argv[])
{
	uint8_t		title ;


	set_sys_clock_khz(200000, true) ;

	setup_default_uart() ;

	common_init() ;

	title = 1 ;

	while( 1 ) {
		g.count1++ ;

		vga_waitVSync() ;
		sound_vsync() ;

		// パッド状態更新
		g.pad.old  = g.pad.norm ;
		g.pad.norm = g_padInfo.status ;
		g.pad.once = (g.pad.norm ^ g.pad.old) & g.pad.norm ;

#ifdef  DEBUG
		// スロー
		if( g.pad.norm & PAD_LB ) {
			if( g.count1 & 15 ) {
				continue ;
			}
		}
#endif//DEBUG

		if( title ) {

			drawString(14, 23, "PUSH BUTTON", 7) ;

			if( g.pad.once & (PAD_SELECT|PAD_START|PAD_BTN( 0 )|PAD_BTN( 1 )|PAD_BTN( 2 )) ) {

				vga_waitVSync() ;
				vga_setTextBlank() ;

				bg_init() ;
				player_create() ;
				bit_create() ;
				bit_create() ;
				force_create() ;

				vga_waitVSync() ;
				vga_setText(&g.vram.txt[0][0], &g.vram.pat[0][0], &g_palData[0][0]) ;

				sound_playBGM( 0 ) ;

				title = 0 ;
			}

		} else {
			bg_draw( &g.vram.txt[0][0] ) ;

			drawStatus() ;

			task_exec( &g.playerList ) ;
			task_exec( &g.enemyList ) ;

			bg_update() ;
		}
	}

	return 0 ;
}
