// -------------------------------------------------------------------------------- 
//!  @file  bit.c                                                                   
//!  ビットタスクのソースファイル                                                   
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "bit.h"


	enum {
		BEAM_MAX					=	2							//! ビーム最大数
	} ;


// -------------------------------------------------------------------------------- 
//!  ビームタスクを破棄                                                             
//!                                                                                 
//!  @param [in,out] p_tcb  タスクコントロールブロック                              
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
static inline void RAMFUNC beam_destroy( TCB *p_tcb )
{
	p_tcb->p_parent->count1++ ;

	task_detach( p_tcb ) ;
	task_releaseTcb( p_tcb ) ;
}


// -------------------------------------------------------------------------------- 
//!  ビームタスク処理                                                               
//!                                                                                 
//!  @param [in,out] p_tcb  タスクコントロールブロック                              
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
static void RAMFUNC beam( TCB *p_tcb )
{
	uint16_t	*p_txt ;
	int8_t		x, y ;

	// 移動
	if( p_tcb->status & TCB_STATUS_LEFT ) {
		p_tcb->bx.n -= toFp( 0.8 ) ;
	} else {
		p_tcb->bx.n += toFp( 0.8 ) ;
	}

	// 壁
	x     = p_tcb->bx.h - g.bg.x.h ;
	y     = p_tcb->by.h - g.bg.y.h ;
	p_txt = &g.vram.txt[y-1][x-(g.forceTcb.status&TCB_STATUS_LEFT)] ;
	if( (p_txt[0]|p_txt[DRAW_WIDTH]) & WALL ) {
		beam_destroy( p_tcb ) ;
		return ;
	}

	obj_updateAction( p_tcb ) ;

	// 描画
	if( obj_drawPattern(x, p_tcb->y.h, p_tcb->pattern) == false ) {
		beam_destroy( p_tcb ) ;
		return ;
	}
}


// -------------------------------------------------------------------------------- 
//!  ビットタスク処理                                                               
//!                                                                                 
//!  @param [in,out] p_tcb  タスクコントロールブロック                              
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void RAMFUNC bit( TCB *p_tcb )
{
	TCB		*p_beam ;

	// 移動
	p_tcb->x.h = g.playerTcb.x.h ;
	p_tcb->y.h = g.playerTcb.y.h + p_tcb->yofs ;

	obj_updateAction( p_tcb ) ;
	obj_drawPattern(p_tcb->x.h, p_tcb->y.h, p_tcb->pattern) ;

	// ビーム
	if( g.pad.once & PAD_BTN( 0 ) ) {
		if( p_tcb->count1 && (g.forceTcb.status&TCB_STATUS_ATTACH) ) {
			p_beam           = task_getTcb() ;
			p_beam->p_parent = p_tcb ;
			p_beam->taskProc = beam ;
			p_beam->status  |= (g.forceTcb.status & TCB_STATUS_LEFT) ;
			p_beam->bx.n     = g.bg.x.n + p_tcb->x.n ;
			p_beam->by.n     = g.bg.y.n + p_tcb->y.n ;

			if( g.forceTcb.status & TCB_STATUS_LEFT ) {
				p_beam->bx.h-- ;
			}

			obj_setAction(p_beam, p_tcb->count0+(p_beam->status&TCB_STATUS_LEFT)) ;

			task_attachPrev(p_tcb, p_beam) ;

			p_tcb->count1-- ;
		}
	}
}


// -------------------------------------------------------------------------------- 
//!  ビットタスクを生成                                                             
//!                                                                                 
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
void RAMFUNC bit_create( void )
{
	TCB		*p_tcb ;
	int8_t	yofs ;


	yofs = 3 ;
	if( !g.bitCount ) {
		yofs = -yofs ;
	}

	p_tcb           = task_getTcb() ;
	p_tcb->taskProc = bit ;
	p_tcb->yofs     = yofs ;
	p_tcb->count0   = ACT_BITBEAM_RU + (g.bitCount*2) ;
	p_tcb->count1   = BEAM_MAX ;

	obj_setAction(p_tcb, ACT_BIT_U+g.bitCount) ;

	task_attachPrev(&g.playerTcb, p_tcb) ;

	g.bitCount++ ;
}
