// -------------------------------------------------------------------------------- 
//!  @file  rlaser.c                                                                
//!  反射レーザータスクのソースファイル                                             
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "rlaser.h"


	enum {
		LASER_LENGTH				=	8,							//!< レーザー長さ
		REFLECT_MAX					=	8							//!< 反射最大数
	} ;


// -------------------------------------------------------------------------------- 
//!  反射レーザータスクを破棄                                                       
//!                                                                                 
//!  @param [in,out] p_tcb  タスクコントロールブロック                              
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void RAMFUNC rlaser_destroy( TCB *p_tcb )
{
	FORCEBEAM	*p_forceBeam ;


	p_forceBeam          = (FORCEBEAM *)p_tcb->p_address ;
	p_forceBeam->remain += p_tcb->count0 ;

	task_detach( p_tcb ) ;
	task_releaseTcb( p_tcb ) ;
}


// -------------------------------------------------------------------------------- 
//!  反射レーザータスク処理                                                         
//!                                                                                 
//!  @param [in,out] p_tcb  タスクコントロールブロック                              
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
static void RAMFUNC rlaser_040( TCB *p_tcb ) ;

static void RAMFUNC rlaser( TCB *p_tcb )
{
	TCB			*p_beam ;
	FORCEBEAM	*p_forceBeam ;


	if( --p_tcb->timer0 ) {
		rlaser_040( p_tcb ) ;
		return ;
	}

	if( --p_tcb->count0 ) {
		p_forceBeam = (FORCEBEAM *)p_tcb->p_address ;

		p_beam            = task_getTcb() ;
		p_beam->taskProc  = rlaser ;
		p_beam->p_address = p_forceBeam ;
		p_beam->angle     = p_tcb->angle ;
		p_beam->count0    = p_tcb->count0 ;
		p_beam->count1    = REFLECT_MAX ;
		p_beam->timer0    = 2 ;
		p_beam->x.h       = p_forceBeam->rbx ;
		p_beam->y.h       = p_forceBeam->rby ;

		task_attachHead(&g.playerList, p_beam) ;
	}

	p_tcb->count0 = 1 ;

	p_tcb->taskProc = rlaser_040 ;

	rlaser_040( p_tcb ) ;
}


static void RAMFUNC rlaser_040( TCB *p_tcb )
{
	static int8_t		add[][2]={
							{  1,  0 },
							{  1,  1 },
							{  0,  1 },
							{ -1,  1 },
							{ -1,  0 },
							{ -1, -1 },
							{  0, -1 },
							{  1, -1 },
						} ;
	static uint16_t		charaCode[]={
							0x5080,
							0x50ef,
							0x5080,
							0x50ee
						} ;
	FI8					bx, by ;
	int8_t				sx, sy, ex, ey ;
	uint16_t			code ;

	// 反射回数オーバー
	if( !p_tcb->count1 ) {
		rlaser_destroy( p_tcb ) ;
		return ;
	}

	// 移動
	bx           = p_tcb->bx ;
	by           = p_tcb->by ;
	p_tcb->bx.h += add[p_tcb->angle/32][0] ;
	p_tcb->by.h += add[p_tcb->angle/32][1] ;

	sx = bx.h        - g.bg.x.h ;
	sy = by.h        - g.bg.y.h ;
	ex = p_tcb->bx.h - g.bg.x.h ;
	ey = p_tcb->by.h - g.bg.y.h ;

	// 以前の位置にレーザー描画
	if( ((uint8_t )sx<DRAW_WIDTH) && ((uint8_t )sy<DRAW_HEIGHT) ) {
		code = charaCode[(p_tcb->angle/32)&3] ;

		// クロス
		if( code >= 0x50ee ) {
			if( (code^1) == g.vram.txt[sy][sx] ) {
				code = 0x50f0 ;
			}
		}
		g.vram.txt[sy][sx] = code ;
	} else {
		rlaser_destroy( p_tcb ) ;
		return ;
	}

	// 反射
	if( g.vram.txt[ey][sx] & WALL ) {			// 上下あり
		if( g.vram.txt[sy][ex] & WALL ) {		// 左右あり
			p_tcb->angle ^= 128 ;							// 反転
			p_tcb->bx     = bx ;
			p_tcb->by     = by ;
			ex            = sx ;
			ey            = sy ;
			p_tcb->count1-- ;
		} else {								// 左右なし
			p_tcb->angle  = -p_tcb->angle ;					// ｙ方向のみ反転
			p_tcb->by     = by ;
			ey            = sy ;
			p_tcb->count1-- ;
		}
	} else {									// 上下なし
		if( g.vram.txt[sy][ex] & WALL ) {		// 左右あり
			p_tcb->angle  = -(p_tcb->angle + 64) - 64 ;		// ｘ方向のみ反転
			p_tcb->bx     = bx ;
			ex            = sx ;
			p_tcb->count1-- ;
		} else
		if( g.vram.txt[ey][ex] & WALL ) {		// 斜めあり
			p_tcb->angle ^= 128 ;							// 反転
			p_tcb->bx     = bx ;
			p_tcb->by     = by ;
			ex            = sx ;
			ey            = sy ;
			p_tcb->count1-- ;
		}
	}

	// 新しい位置にレーザー描画
	if( ((uint8_t )ex<DRAW_WIDTH) && ((uint8_t )ey<DRAW_HEIGHT) ) {
		code = charaCode[(p_tcb->angle/32)&3] ;

		// クロス
		if( code >= 0x50ee ) {
			if( (code^1) == g.vram.txt[ey][ex] ) {
				code = 0x50f0 ;
			}
		}
		g.vram.txt[ey][ex] = code ;
	}
}


// -------------------------------------------------------------------------------- 
//!  反射レーザータスクを生成                                                       
//!                                                                                 
//!  @param [in,out] p_force  フォースタスク TCB                                    
//!  @return                  なし                                                  
// -------------------------------------------------------------------------------- 
void RAMFUNC rlaser_create( TCB *p_force )
{
	TCB			*p_tcb ;
	FORCEBEAM	*p_forceBeam ;
	int8_t		bx, by ;
	uint16_t	*p_txt ;
	uint32_t	angle ;
	int			index ;


	if( !(g.pad.once&PAD_BTN( 0 )) ) {
		return ;
	}

	if( p_force->status & TCB_STATUS_LEFT ) {	// 左
		bx    = p_force->x.h - 1 ;
		by    = p_force->y.h - 1 ;
		angle = (160 << 24) |
		        (128 << 16) |
		        ( 96 <<  8) ;
	} else {									// 右
		bx    = p_force->x.h ;
		by    = p_force->y.h - 1 ;
		angle = (224 << 24) |
		        (  0 << 16) |
		        ( 32 <<  8) ;
	}

	p_forceBeam = &g.forceBeam[0] ;
	for(index=0; index<3; index++,p_forceBeam++,angle<<=8) {

		// ビーム数が足りないなら無視
		if( p_forceBeam->remain < 8 ) {
			continue ;
		}

		// 壁の中は無視
		p_txt = &g.vram.txt[by-1][bx] ;
		if( (p_txt[0]|p_txt[SCREEN_WIDTH]) & WALL ) {
			continue ;
		}

		p_forceBeam->remain = 0 ;
		p_forceBeam->rbx    = g.bg.x.h +  bx ;
		p_forceBeam->rby    = g.bg.y.h + (by + ((index / 2) & 1)) ;

		p_tcb            = task_getTcb() ;
		p_tcb->taskProc  = rlaser ;
		p_tcb->p_address = p_forceBeam ;
		p_tcb->angle     = (uint8_t )(angle >> 24) ;
		p_tcb->count0    = LASER_LENGTH ;
		p_tcb->count1    = REFLECT_MAX ;
		p_tcb->timer0    = 2 ;
		p_tcb->x.h       = p_forceBeam->rbx ;
		p_tcb->y.h       = p_forceBeam->rby ;

		task_attachHead(&g.playerList, p_tcb) ;
	}
}
