// -------------------------------------------------------------------------------- 
//!  @file  task.c                                                                  
//!  タスク管理のソースファイル                                                     
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"


static TCB							*m_p_nonExistentTcb ;			//!< 未登録状態タスクリスト
static TCB							m_tcb[TASK_MAX] ;				//!< タスクコントロールブロック


// -------------------------------------------------------------------------------- 
//!  初期化                                                                         
//!                                                                                 
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
void task_init( void )
{
	TCB		*p_tcb, *p_tcbEnd ;

	// 未登録状態タスクリスト作成
	m_p_nonExistentTcb = &m_tcb[0] ;
	p_tcb              = &m_tcb[0] ;
	p_tcbEnd           = &m_tcb[TASK_MAX-1] ;

	do {
		p_tcb[0].p_next = &p_tcb[1] ;
	} while( ++p_tcb != p_tcbEnd ) ;

	p_tcb->p_next = NULL ;
}


// -------------------------------------------------------------------------------- 
//!  TCB を取得                                                                     
//!                                                                                 
//!  @return TCB                                                                    
// -------------------------------------------------------------------------------- 
TCB * RAMFUNC task_getTcb( void )
{
	TCB		*p_tcb ;


	p_tcb              = m_p_nonExistentTcb ;
	m_p_nonExistentTcb = p_tcb->p_next ;

	p_tcb->status = 0 ;

	return p_tcb ;
}


// -------------------------------------------------------------------------------- 
//!  TCB を解放                                                                     
//!                                                                                 
//!  @param [out] p_tcb  TCB                                                        
//!  @return             なし                                                       
// -------------------------------------------------------------------------------- 
void RAMFUNC task_releaseTcb( TCB *p_tcb )
{
	p_tcb->p_next      = m_p_nonExistentTcb ;
	m_p_nonExistentTcb = p_tcb ;
}


// -------------------------------------------------------------------------------- 
//!  指定タスクの前へタスクを取り付ける                                             
//!                                                                                 
//!  @param [in,out] p_dest  取り付け先 TCB                                         
//!  @param [out]    p_srce  取り付け元 TCB                                         
//!  @return                 なし                                                   
// -------------------------------------------------------------------------------- 
void RAMFUNC task_attachPrev(TCB *p_dest, TCB *p_srce)
{
	TCB		*p_prev ;


	p_prev = p_dest->p_prev ;

	p_srce->p_prev = p_prev ;
	p_srce->p_next = p_dest ;
	p_prev->p_next = p_srce ;
	p_dest->p_prev = p_srce ;
}


// -------------------------------------------------------------------------------- 
//!  指定タスクの次へタスクを取り付ける                                             
//!                                                                                 
//!  @param [in,out] p_dest  取り付け先 TCB                                         
//!  @param [out]    p_srce  取り付け元 TCB                                         
//!  @return                 なし                                                   
// -------------------------------------------------------------------------------- 
void RAMFUNC task_attachNext(TCB *p_dest, TCB *p_srce)
{
	TCB		*p_next ;


	p_next = p_dest->p_next ;

	p_srce->p_prev = p_dest ;
	p_srce->p_next = p_next ;
	p_next->p_prev = p_srce ;
	p_dest->p_next = p_srce ;
}


// -------------------------------------------------------------------------------- 
//!  タスクを取り外す                                                               
//!                                                                                 
//!  @param [in,out] p_tcb  TCB                                                     
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
void RAMFUNC task_detach( TCB *p_tcb )
{
	TCB		*p_prev, *p_next ;


	p_prev         = p_tcb->p_prev ;
	p_next         = p_tcb->p_next ;
	p_prev->p_next = p_next ;
	p_next->p_prev = p_prev ;
}


// -------------------------------------------------------------------------------- 
//!  タスクを実行                                                                   
//!                                                                                 
//!  @param [in,out] p_list  TCB リスト                                             
//!  @return                 なし                                                   
// -------------------------------------------------------------------------------- 
void RAMFUNC task_exec( TCBLIST *p_list )
{
	TCB		*p_tcb, *p_next ;


	p_tcb = p_list->p_next ;

	while( p_tcb != (TCB *)p_list ) {
		p_next = p_tcb->p_next ;

		p_tcb->taskProc( p_tcb ) ;

		p_tcb = p_next ;
	}
}
