// -------------------------------------------------------------------------------- 
//!  @file  task.h                                                                  
//!  タスク管理のヘッダファイル                                                     
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#ifndef __TASK_H__INCLUDED__
#define __TASK_H__INCLUDED__


typedef struct _tcb					TCB ;							//!< タスクコントロールブロック
typedef void (*TASKPROC)( TCB * ) ;									//!< タスク処理

	//! タスクコントロールブロック状態
	enum {
		TCB_STATUS_LEFT				=	1 << 0,						//!< 左向き
		TCB_STATUS_ATTACH			=	1 << 1,						//!< フォース取り付け中
		TCB_STATUS_WALL				=	1 << 2,						//!< 壁当たり中
		TCB_STATUS_ACTEND			=	1 << 7,						//!< アクション終了
	} ;

	//! タスクコントロールブロックリスト
	typedef struct {
		TCB							*p_prev ;						//!< 前タスク TCB
		TCB							*p_next ;						//!< 次タスク TCB
	} TCBLIST ;

	//! タスクコントロールブロック
	struct _tcb {
		TCB							*p_prev ;						//!< 前タスク TCB
		TCB							*p_next ;						//!< 次タスク TCB

		union {
			TCB						*p_parent ;						//!< 親タスク TCB
			TCB						*p_child ;						//!< 子タスク TCB
			void					*p_address ;					//!< 汎用アドレス
		} ;

		TASKPROC					taskProc ;						//!< タスク処理
		uint8_t						status ;						//!< 状態
		uint8_t						count0 ;						//!< 汎用カウンタ０
		uint8_t						count1 ;						//!< 汎用カウンタ１
		uint8_t						timer0 ;						//!< 汎用タイマー０
		uint8_t						const *p_action ;				//!< アクションデータ
		uint8_t						pattern ;						//!< パターン番号
		uint8_t						patternCount ;					//!< パターン表示カウンタ

		union {
			FI8						x ;								//!< 画面ｘ座標
			FI8						bx ;							//!< 背景ｘ座標
		} ;

		union {
			FI8						y ;								//!< 画面ｙ座標
			FI8						by ;							//!< 背景ｙ座標
		} ;

		union {
			int8_t					xofs ;							//!< ｘ座標オフセット
			uint8_t					speed ;							//!< 速度
		} ;

		union {
			int8_t					yofs ;							//!< ｙ座標オフセット
			uint8_t					angle ;							//!< 角度
		} ;
	} ;


extern void          task_init( void ) ;
extern TCB * RAMFUNC task_getTcb( void ) ;
extern void  RAMFUNC task_releaseTcb( TCB *p_tcb ) ;
extern void  RAMFUNC task_attachPrev(TCB *p_dest, TCB *p_srce) ;
extern void  RAMFUNC task_attachNext(TCB *p_dest, TCB *p_srce) ;
extern void  RAMFUNC task_detach( TCB *p_tcb ) ;
extern void  RAMFUNC task_exec( TCBLIST *p_list ) ;


// -------------------------------------------------------------------------------- 
//!  タスクコントロールブロックリストを初期化                                       
//!                                                                                 
//!  @param [out] p_list  タスクコントロールブロックリスト                          
//!  @return              なし                                                      
// -------------------------------------------------------------------------------- 
inline void task_initList( TCBLIST *p_list )
{
	p_list->p_prev = (TCB *)p_list ;
	p_list->p_next = (TCB *)p_list ;
}


// -------------------------------------------------------------------------------- 
//!  タスクコントロールブロックリストの先頭へ取り付ける                             
//!                                                                                 
//!  @param [in,out] p_list  タスクコントロールブロックリスト                       
//!  @param [out]    p_tcb   タスクコントロールブロック                             
//!  @return                 なし                                                   
// -------------------------------------------------------------------------------- 
inline void task_attachHead(TCBLIST *p_list, TCB *p_tcb)
{
	task_attachNext((TCB *)p_list, p_tcb) ;
}


// -------------------------------------------------------------------------------- 
//!  タスクコントロールブロックリストの最後へ取り付ける                             
//!                                                                                 
//!  @param [in,out] p_list  タスクコントロールブロックリスト                       
//!  @param [out]    p_tcb   タスクコントロールブロック                             
//!  @return                 なし                                                   
// -------------------------------------------------------------------------------- 
inline void task_attachTail(TCBLIST *p_list, TCB *p_tcb)
{
	task_attachPrev((TCB *)p_list, p_tcb) ;
}


#endif//__TASK_H__INCLUDED__
