// -------------------------------------------------------------------------------- 
//!  @file  config.h                                                                
//!  設定ファイル                                                                   
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#ifndef __CONFIG_H__INCLUDED__
#define __CONFIG_H__INCLUDED__


#define DEBUG														//!< デバッグ
#define BITMAP_MODE					8								//!< ビットマップモード（4:4bit, 8:8bit）

//! RAM 関数
#define RAMFUNC						__attribute__ ((section(".data.ram_funcs")))

// RP2040
#define RP2040_CFG_SYSCLK			200								//!< システムクロック周波数（MHz）
																	//   VGA 使用時は libvga/vga/vga.pio 内の sysclk へこの値を記述する必要あり

// VGA
#define VGA_CFG_DOTCLK				(RP2040_CFG_SYSCLK / 25)		//!< １ドットクロックあたりのクロック数
#define VGA_CFG_PIN_HSYNC			21								//!< HSYNC ピン番号
#define VGA_CFG_PIN_VSYNC			20								//!< VSYNC ピン番号
#define VGA_CFG_PIN_PIXEL			2								//!< ピクセルピン開始番号
#define VGA_CFG_PIN_PIXELCOUNT		12								//!< ピクセルビット数
#define VGA_CFG_PIO					1								//!< PIO 番号
#define VGA_CFG_DMA					1								//!< DMA 番号
#define VGA_CFG_DMACH_PIXEL			0								//!< ピクセル転送用 DMA チャンネル番号
#define VGA_CFG_DMACH_DATA			1								//!< データ転送用 DMA チャンネル番号
#define VGA_CFG_PAD													//!< 空き時間にパッドを更新
#define VGA_CFG_BMP4												//!<  4bit ビットマップ面使用
#define VGA_CFG_BMP8												//!<  8bit ビットマップ面使用
//#define VGA_CFG_BMP16												//!< 16bit ビットマップ面使用
//#define VGA_CFG_BGTILE												//!< 背景タイル面使用
//#define VGA_CFG_FGTILE												//!< 前景タイル面使用
//#define VGA_CFG_SPR													//!< スプライト面使用
//#define VGA_CFG_SPR_MAX				64								//!< スプライト最大数
//#define VGA_CFG_SPR_LINE_MAX		20								//!< １行あたりのスプライト表示最大数
#define VGA_CFG_TXT													//!< テキスト面使用

// サウンド
//#define SOUNDDAC_CFG_TIME			50								//!< 周期時間（usec）
//#define SOUNDDAC_CFG_DAC_MCP49x1									//!< MCP49x1 ドライバ使用

// パッド
#define PAD_CFG_ID					-1								//!< -1=自動

// MCP49x1 ドライバ
//#define MCP49x1_CFG_PIN_CS			22								//!< CS ピン番号
//#define MCP49x1_CFG_PIN_SCK			26								//!< SCK ピン番号
//#define MCP49x1_CFG_PIN_SDI			27								//!< SDI ピン番号
//#define MCP49x1_CFG_PIN_LDAC		28								//!< LDAC ピン番号
//#define MCP49x1_CFG_SPI				spi1							//!< SPI レジスタ
//#define MCP49x1_CFG_SPI_CLK			20000000						//!< SPI クロック周波数
//#define MCP49x1_CFG_COMMAND			0x3000							//!< コマンドビット

// PWM - SN76489
//#define PWMSN76489_CFG_BASECLK			(3579545 / 32)				//!< ベースクロック
//#define PWMSN76489_CFG_PIN_TONE0		22							//!< トーン０ピン番号
//#define PWMSN76489_CFG_TONE0_SLICE		3							//!< トーン０ PWM スライス
//#define PWMSN76489_CFG_TONE0_CHANNEL	PWM_CHAN_A					//!< トーン０ PWM チャンネル
//#define PWMSN76489_CFG_PIN_TONE1		26							//!< トーン１ピン番号
//#define PWMSN76489_CFG_TONE1_SLICE		5							//!< トーン１ PWM スライス
//#define PWMSN76489_CFG_TONE1_CHANNEL	PWM_CHAN_A					//!< トーン１ PWM チャンネル
//#define PWMSN76489_CFG_PIN_TONE2		28							//!< トーン２ピン番号
//#define PWMSN76489_CFG_TONE2_SLICE		6							//!< トーン２ PWM スライス
//#define PWMSN76489_CFG_TONE2_CHANNEL	PWM_CHAN_A					//!< トーン２ PWM チャンネル
//#define PWMSN76489_CFG_PIN_NOISE		27							//!< ノイズピン番号

// PCM - SN76489
//#define PCMSN76489_CFG_BASECLK		3579545							//!< ベースクロック
//#define PCMSN76489_CFG_TIME			SOUNDDAC_CFG_TIME				//!< 更新時間（usec）

// PCM - SCC
//#define PCMSCC_CFG_BASECLK			3579545							//!< ベースクロック
//#define PCMSCC_CFG_TIME				SOUNDDAC_CFG_TIME				//!< 更新時間（usec）

// PCM - VRC6
//#define PCMVRC6_CFG_BASECLK			1789773							//!< ベースクロック
//#define PCMVRC6_CFG_TIME			SOUNDDAC_CFG_TIME				//!< 更新時間（usec）

// DPCM3
//#define DPCM3_CFG_CHANNEL_COUNT		4								//!< チャンネル数
//#define DPCM3_CFG_ADJUSTVOLUME		5								//!< ボリューム調整シフト数

// シリアル通信パッド
#define PAD_SERIAL_CFG_BAUDRATE		115200							//!< ボーレート
#define PAD_SERIAL_CFG_PIN_TX		0								//!< 送信ピン番号（未使用）
#define PAD_SERIAL_CFG_PIN_RX		17								//!< 受信ピン番号
#define PAD_SERIAL_CFG_UART			uart0							//!< UART レジスタ
#define PAD_SERIAL_CFG_UART_IRQ		UART0_IRQ						//!< UART 割り込み番号

// PC エンジンパッド
#define PAD_PCE_CFG_PIN_SEL			1								//!< SEL ピン番号
#define PAD_PCE_CFG_PIN_CLR			18								//!< CLR ピン番号
#define PAD_PCE_CFG_PIN_D0			14								//!< D0 ピン番号
#define PAD_PCE_CFG_PIN_D1			15								//!< D1 ピン番号
#define PAD_PCE_CFG_PIN_D2			16								//!< D2 ピン番号
#define PAD_PCE_CFG_PIN_D3			17								//!< D3 ピン番号

// メガドライブパッド
#define PAD_MD_CFG_PIN_SEL			1								//!< SEL ピン番号
#define PAD_MD_CFG_PIN_D0			14								//!< D0 ピン番号
#define PAD_MD_CFG_PIN_D1			15								//!< D1 ピン番号
#define PAD_MD_CFG_PIN_D2			16								//!< D2 ピン番号
#define PAD_MD_CFG_PIN_D3			17								//!< D3 ピン番号
#define PAD_MD_CFG_PIN_D4			18								//!< D4 ピン番号
#define PAD_MD_CFG_PIN_D5			19								//!< D5 ピン番号

// スーパーファミコンパッド
#define PAD_SFC_CFG_PIN_CLK			14								//!< CLK ピン番号
#define PAD_SFC_CFG_PIN_PS			15								//!< P/S ピン番号
#define PAD_SFC_CFG_PIN_DAT1		16								//!< DAT1 ピン番号
#define PAD_SFC_CFG_PIN_DAT2		17								//!< DAT2 ピン番号
#define PAD_SFC_CFG_PIN_SELECT		18								//!< SELECT ピン番号

// PS コントローラーパッド
#define PAD_PS_CFG_PIN_ACK			1								//!< ACK ピン番号
#define PAD_PS_CFG_PIN_MISO			16								//!< MOSI ピン番号
#define PAD_PS_CFG_PIN_SS			17								//!< SS ピン番号
#define PAD_PS_CFG_PIN_SCLK			18								//!< SCLK ピン番号
#define PAD_PS_CFG_PIN_MOSI			19								//!< MOSI ピン番号
#define PAD_PS_CFG_SPI				spi0							//!< SPI レジスタ

// クラシックコントローラ
#define PAD_CLASSIC_CFG_PIN_DETECT	14								//!< DETECT ピン番号
#define PAD_CLASSIC_CFG_PIN_SDA		18								//!< SDA ピン番号
#define PAD_CLASSIC_CFG_PIN_SCL		19								//!< SCL ピン番号
#define PAD_CLASSIC_CFG_I2C			i2c1							//!< I2C レジスタ
#define PAD_CLASSIC_CFG_I2C_SPEED	350000							//!< I2C 速度


#endif//__CONFIG_H__INCLUDED__
